/* Generated by Cython 0.18 on Mon Dec  9 16:00:33 2013 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02040000
    #error Cython requires Python 2.4+.
#else
#include <stddef.h> /* For offsetof */
#ifndef offsetof
#define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
#define CYTHON_COMPILING_IN_PYPY 1
#define CYTHON_COMPILING_IN_CPYTHON 0
#else
#define CYTHON_COMPILING_IN_PYPY 0
#define CYTHON_COMPILING_IN_CPYTHON 1
#endif
#if PY_VERSION_HEX < 0x02050000
  typedef int Py_ssize_t;
  #define PY_SSIZE_T_MAX INT_MAX
  #define PY_SSIZE_T_MIN INT_MIN
  #define PY_FORMAT_SIZE_T ""
  #define CYTHON_FORMAT_SSIZE_T ""
  #define PyInt_FromSsize_t(z) PyInt_FromLong(z)
  #define PyInt_AsSsize_t(o)   __Pyx_PyInt_AsInt(o)
  #define PyNumber_Index(o)    ((PyNumber_Check(o) && !PyFloat_Check(o)) ? PyNumber_Int(o) : \
                                (PyErr_Format(PyExc_TypeError, \
                                              "expected index value, got %.200s", Py_TYPE(o)->tp_name), \
                                 (PyObject*)0))
  #define __Pyx_PyIndex_Check(o) (PyNumber_Check(o) && !PyFloat_Check(o) && \
                                  !PyComplex_Check(o))
  #define PyIndex_Check __Pyx_PyIndex_Check
  #define PyErr_WarnEx(category, message, stacklevel) PyErr_Warn(category, message)
  #define __PYX_BUILD_PY_SSIZE_T "i"
#else
  #define __PYX_BUILD_PY_SSIZE_T "n"
  #define CYTHON_FORMAT_SSIZE_T "z"
  #define __Pyx_PyIndex_Check PyIndex_Check
#endif
#if PY_VERSION_HEX < 0x02060000
  #define Py_REFCNT(ob) (((PyObject*)(ob))->ob_refcnt)
  #define Py_TYPE(ob)   (((PyObject*)(ob))->ob_type)
  #define Py_SIZE(ob)   (((PyVarObject*)(ob))->ob_size)
  #define PyVarObject_HEAD_INIT(type, size) \
          PyObject_HEAD_INIT(type) size,
  #define PyType_Modified(t)
  typedef struct {
     void *buf;
     PyObject *obj;
     Py_ssize_t len;
     Py_ssize_t itemsize;
     int readonly;
     int ndim;
     char *format;
     Py_ssize_t *shape;
     Py_ssize_t *strides;
     Py_ssize_t *suboffsets;
     void *internal;
  } Py_buffer;
  #define PyBUF_SIMPLE 0
  #define PyBUF_WRITABLE 0x0001
  #define PyBUF_FORMAT 0x0004
  #define PyBUF_ND 0x0008
  #define PyBUF_STRIDES (0x0010 | PyBUF_ND)
  #define PyBUF_C_CONTIGUOUS (0x0020 | PyBUF_STRIDES)
  #define PyBUF_F_CONTIGUOUS (0x0040 | PyBUF_STRIDES)
  #define PyBUF_ANY_CONTIGUOUS (0x0080 | PyBUF_STRIDES)
  #define PyBUF_INDIRECT (0x0100 | PyBUF_STRIDES)
  #define PyBUF_RECORDS (PyBUF_STRIDES | PyBUF_FORMAT | PyBUF_WRITABLE)
  #define PyBUF_FULL (PyBUF_INDIRECT | PyBUF_FORMAT | PyBUF_WRITABLE)
  typedef int (*getbufferproc)(PyObject *, Py_buffer *, int);
  typedef void (*releasebufferproc)(PyObject *, Py_buffer *);
#endif
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos) \
          PyCode_New(a, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos) \
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
#if PY_MAJOR_VERSION < 3 && PY_MINOR_VERSION < 6
  #define PyUnicode_FromString(s) PyUnicode_Decode(s, strlen(s), "UTF-8", "strict")
#endif
#if PY_MAJOR_VERSION >= 3
  #define Py_TPFLAGS_CHECKTYPES 0
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#if (PY_VERSION_HEX < 0x02060000) || (PY_MAJOR_VERSION >= 3)
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ? \
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
#else
  #define CYTHON_PEP393_ENABLED 0
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((k=k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_VERSION_HEX < 0x02060000
  #define PyBytesObject                PyStringObject
  #define PyBytes_Type                 PyString_Type
  #define PyBytes_Check                PyString_Check
  #define PyBytes_CheckExact           PyString_CheckExact
  #define PyBytes_FromString           PyString_FromString
  #define PyBytes_FromStringAndSize    PyString_FromStringAndSize
  #define PyBytes_FromFormat           PyString_FromFormat
  #define PyBytes_DecodeEscape         PyString_DecodeEscape
  #define PyBytes_AsString             PyString_AsString
  #define PyBytes_AsStringAndSize      PyString_AsStringAndSize
  #define PyBytes_Size                 PyString_Size
  #define PyBytes_AS_STRING            PyString_AS_STRING
  #define PyBytes_GET_SIZE             PyString_GET_SIZE
  #define PyBytes_Repr                 PyString_Repr
  #define PyBytes_Concat               PyString_Concat
  #define PyBytes_ConcatAndDel         PyString_ConcatAndDel
#endif
#if PY_VERSION_HEX < 0x02060000
  #define PySet_Check(obj)             PyObject_TypeCheck(obj, &PySet_Type)
  #define PyFrozenSet_Check(obj)       PyObject_TypeCheck(obj, &PyFrozenSet_Type)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_VERSION_HEX < 0x03020000
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if (PY_MAJOR_VERSION < 3) || (PY_VERSION_HEX >= 0x03010300)
  #define __Pyx_PySequence_GetSlice(obj, a, b) PySequence_GetSlice(obj, a, b)
  #define __Pyx_PySequence_SetSlice(obj, a, b, value) PySequence_SetSlice(obj, a, b, value)
  #define __Pyx_PySequence_DelSlice(obj, a, b) PySequence_DelSlice(obj, a, b)
#else
  #define __Pyx_PySequence_GetSlice(obj, a, b) (unlikely(!(obj)) ? \
        (PyErr_SetString(PyExc_SystemError, "null argument to internal routine"), (PyObject*)0) : \
        (likely((obj)->ob_type->tp_as_mapping) ? (PySequence_GetSlice(obj, a, b)) : \
            (PyErr_Format(PyExc_TypeError, "'%.200s' object is unsliceable", (obj)->ob_type->tp_name), (PyObject*)0)))
  #define __Pyx_PySequence_SetSlice(obj, a, b, value) (unlikely(!(obj)) ? \
        (PyErr_SetString(PyExc_SystemError, "null argument to internal routine"), -1) : \
        (likely((obj)->ob_type->tp_as_mapping) ? (PySequence_SetSlice(obj, a, b, value)) : \
            (PyErr_Format(PyExc_TypeError, "'%.200s' object doesn't support slice assignment", (obj)->ob_type->tp_name), -1)))
  #define __Pyx_PySequence_DelSlice(obj, a, b) (unlikely(!(obj)) ? \
        (PyErr_SetString(PyExc_SystemError, "null argument to internal routine"), -1) : \
        (likely((obj)->ob_type->tp_as_mapping) ? (PySequence_DelSlice(obj, a, b)) : \
            (PyErr_Format(PyExc_TypeError, "'%.200s' object doesn't support slice deletion", (obj)->ob_type->tp_name), -1)))
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#endif
#if PY_VERSION_HEX < 0x02050000
  #define __Pyx_GetAttrString(o,n)   PyObject_GetAttrString((o),((char *)(n)))
  #define __Pyx_SetAttrString(o,n,a) PyObject_SetAttrString((o),((char *)(n)),(a))
  #define __Pyx_DelAttrString(o,n)   PyObject_DelAttrString((o),((char *)(n)))
#else
  #define __Pyx_GetAttrString(o,n)   PyObject_GetAttrString((o),(n))
  #define __Pyx_SetAttrString(o,n,a) PyObject_SetAttrString((o),(n),(a))
  #define __Pyx_DelAttrString(o,n)   PyObject_DelAttrString((o),(n))
#endif
#if PY_VERSION_HEX < 0x02050000
  #define __Pyx_NAMESTR(n) ((char *)(n))
  #define __Pyx_DOCSTR(n)  ((char *)(n))
#else
  #define __Pyx_NAMESTR(n) (n)
  #define __Pyx_DOCSTR(n)  (n)
#endif


#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
#define _USE_MATH_DEFINES
#endif
#include <math.h>
#define __PYX_HAVE__h5py___proxy
#define __PYX_HAVE_API__h5py___proxy
#include "stdlib.h"
#include "string.h"
#include "time.h"
#include "unistd.h"
#include "stdint.h"
#include "api_compat.h"
#include "lzf_filter.h"
#include "hdf5.h"
#include "H5Cpublic.h"
#include "hdf5_hl.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifndef CYTHON_INLINE
  #if defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
typedef struct {PyObject **p; char *s; const long n; const char* encoding; const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry; /*proto*/

#define __Pyx_PyBytes_FromUString(s) PyBytes_FromString((char*)s)
#define __Pyx_PyBytes_AsUString(s)   ((unsigned char*) PyBytes_AsString(s))
#define __Pyx_Owned_Py_None(b) (Py_INCREF(Py_None), Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? (Py_INCREF(Py_True), Py_True) : (Py_INCREF(Py_False), Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
static CYTHON_INLINE size_t __Pyx_PyInt_AsSize_t(PyObject*);
#if CYTHON_COMPILING_IN_CPYTHON
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))


#ifdef __GNUC__
  /* Test for GCC > 2.95 */
  #if __GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95))
    #define likely(x)   __builtin_expect(!!(x), 1)
    #define unlikely(x) __builtin_expect(!!(x), 0)
  #else /* __GNUC__ > 2 ... */
    #define likely(x)   (x)
    #define unlikely(x) (x)
  #endif /* __GNUC__ > 2 ... */
#else /* __GNUC__ */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
    
static PyObject *__pyx_m;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "_proxy.pyx",
};

/*--- Type declarations ---*/
struct __pyx_t_4h5py_6_proxy_h5py_scatter_t;
typedef struct __pyx_t_4h5py_6_proxy_h5py_scatter_t __pyx_t_4h5py_6_proxy_h5py_scatter_t;

/* "h5py/_proxy.pyx":16
 * """
 * 
 * cdef enum copy_dir:             # <<<<<<<<<<<<<<
 *     H5PY_SCATTER = 0,
 *     H5PY_GATHER
 */
enum __pyx_t_4h5py_6_proxy_copy_dir {
  __pyx_e_4h5py_6_proxy_H5PY_SCATTER = 0,
  __pyx_e_4h5py_6_proxy_H5PY_GATHER
};

/* "h5py/_proxy.pyx":239
 * # Scatter/gather routines
 * 
 * ctypedef struct h5py_scatter_t:             # <<<<<<<<<<<<<<
 *     size_t i
 *     size_t elsize
 */
struct __pyx_t_4h5py_6_proxy_h5py_scatter_t {
  size_t i;
  size_t elsize;
  void *buf;
};
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname); /*proto*/
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil) \
          if (acquire_gil) { \
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure(); \
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__); \
              PyGILState_Release(__pyx_gilstate_save); \
          } else { \
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__); \
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil) \
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext() \
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif /* CYTHON_REFNANNY */
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

static PyObject *__Pyx_GetName(PyObject *dict, PyObject *name); /*proto*/

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb); /*proto*/
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb); /*proto*/

static CYTHON_INLINE int __Pyx_PySequence_Contains(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause); /*proto*/

static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb); /*proto*/

static CYTHON_INLINE void __Pyx_ExceptionSave(PyObject **type, PyObject **value, PyObject **tb); /*proto*/
static void __Pyx_ExceptionReset(PyObject *type, PyObject *value, PyObject *tb); /*proto*/

static int __Pyx_Print(PyObject*, PyObject *, int); /*proto*/
#if CYTHON_COMPILING_IN_PYPY || PY_MAJOR_VERSION >= 3
static PyObject* __pyx_print = 0;
static PyObject* __pyx_print_kwargs = 0;
#endif

static int __Pyx_PrintOne(PyObject* stream, PyObject *o); /*proto*/

static CYTHON_INLINE unsigned char __Pyx_PyInt_AsUnsignedChar(PyObject *);

static CYTHON_INLINE unsigned short __Pyx_PyInt_AsUnsignedShort(PyObject *);

static CYTHON_INLINE unsigned int __Pyx_PyInt_AsUnsignedInt(PyObject *);

static CYTHON_INLINE char __Pyx_PyInt_AsChar(PyObject *);

static CYTHON_INLINE short __Pyx_PyInt_AsShort(PyObject *);

static CYTHON_INLINE int __Pyx_PyInt_AsInt(PyObject *);

static CYTHON_INLINE signed char __Pyx_PyInt_AsSignedChar(PyObject *);

static CYTHON_INLINE signed short __Pyx_PyInt_AsSignedShort(PyObject *);

static CYTHON_INLINE signed int __Pyx_PyInt_AsSignedInt(PyObject *);

static CYTHON_INLINE int __Pyx_PyInt_AsLongDouble(PyObject *);

static CYTHON_INLINE unsigned long __Pyx_PyInt_AsUnsignedLong(PyObject *);

static CYTHON_INLINE unsigned PY_LONG_LONG __Pyx_PyInt_AsUnsignedLongLong(PyObject *);

static CYTHON_INLINE long __Pyx_PyInt_AsLong(PyObject *);

static CYTHON_INLINE PY_LONG_LONG __Pyx_PyInt_AsLongLong(PyObject *);

static CYTHON_INLINE signed long __Pyx_PyInt_AsSignedLong(PyObject *);

static CYTHON_INLINE signed PY_LONG_LONG __Pyx_PyInt_AsSignedLongLong(PyObject *);

static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename); /*proto*/

static int __Pyx_check_binary_version(void);

static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig); /*proto*/

#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

static PyObject *__Pyx_ImportModule(const char *name); /*proto*/

static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig); /*proto*/

typedef struct {
    int code_line;
    PyCodeObject* code_object;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename); /*proto*/

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t); /*proto*/


/* Module declarations from 'h5py.api_types_ext' */

/* Module declarations from 'h5py.api_types_hdf5' */

/* Module declarations from 'h5py.defs' */
static hid_t (*__pyx_f_4h5py_4defs_H5Dget_space)(hid_t); /*proto*/
static hid_t (*__pyx_f_4h5py_4defs_H5Dget_type)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Dread)(hid_t, hid_t, hid_t, hid_t, hid_t, void *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Dwrite)(hid_t, hid_t, hid_t, hid_t, hid_t, void *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Dvlen_reclaim)(hid_t, hid_t, hid_t, void *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Diterate)(void *, hid_t, hid_t, H5D_operator_t, void *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Sclose)(hid_t); /*proto*/
static hid_t (*__pyx_f_4h5py_4defs_H5Screate_simple)(int, hsize_t *, hsize_t *); /*proto*/
static hssize_t (*__pyx_f_4h5py_4defs_H5Sget_select_npoints)(hid_t); /*proto*/
static hid_t (*__pyx_f_4h5py_4defs_H5Tcreate)(enum H5T_class_t, size_t); /*proto*/
static enum H5T_class_t (*__pyx_f_4h5py_4defs_H5Tget_class)(hid_t); /*proto*/
static size_t (*__pyx_f_4h5py_4defs_H5Tget_size)(hid_t); /*proto*/
static hid_t (*__pyx_f_4h5py_4defs_H5Tget_super)(hid_t); /*proto*/
static htri_t (*__pyx_f_4h5py_4defs_H5Tdetect_class)(hid_t, enum H5T_class_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Tclose)(hid_t); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Tconvert)(hid_t, hid_t, size_t, void *, void *, hid_t); /*proto*/
static htri_t (*__pyx_f_4h5py_4defs_H5Tis_variable_str)(hid_t); /*proto*/
static int (*__pyx_f_4h5py_4defs_H5Tget_nmembers)(hid_t); /*proto*/
static char *(*__pyx_f_4h5py_4defs_H5Tget_member_name)(hid_t, unsigned int); /*proto*/
static hid_t (*__pyx_f_4h5py_4defs_H5Tget_member_type)(hid_t, unsigned int); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Tinsert)(hid_t, char *, size_t, hid_t); /*proto*/
static char *(*__pyx_f_4h5py_4defs_H5Tget_tag)(hid_t); /*proto*/
static H5T_conv_t (*__pyx_f_4h5py_4defs_H5Tfind)(hid_t, hid_t, H5T_cdata_t **); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Aread)(hid_t, hid_t, void *); /*proto*/
static herr_t (*__pyx_f_4h5py_4defs_H5Awrite)(hid_t, hid_t, void *); /*proto*/
static hid_t (*__pyx_f_4h5py_4defs_H5Aget_space)(hid_t); /*proto*/
static hid_t (*__pyx_f_4h5py_4defs_H5Aget_type)(hid_t); /*proto*/

/* Module declarations from 'h5py._proxy' */
static herr_t __pyx_f_4h5py_6_proxy_H5PY_H5Dread(hid_t, hid_t, hid_t, hid_t, hid_t, void *); /*proto*/
static herr_t __pyx_f_4h5py_6_proxy_H5PY_H5Dwrite(hid_t, hid_t, hid_t, hid_t, hid_t, void *); /*proto*/
static hid_t __pyx_f_4h5py_6_proxy_make_reduced_type(hid_t, hid_t); /*proto*/
static void *__pyx_f_4h5py_6_proxy_create_buffer(size_t, size_t, size_t); /*proto*/
static herr_t __pyx_f_4h5py_6_proxy_h5py_scatter_cb(void *, hid_t, unsigned int, hsize_t *, void *); /*proto*/
static herr_t __pyx_f_4h5py_6_proxy_h5py_gather_cb(void *, hid_t, unsigned int, hsize_t *, void *); /*proto*/
static herr_t __pyx_f_4h5py_6_proxy_h5py_copy(hid_t, hid_t, void *, void *, enum __pyx_t_4h5py_6_proxy_copy_dir); /*proto*/
static htri_t __pyx_f_4h5py_6_proxy_needs_bkg_buffer(hid_t, hid_t); /*proto*/
static htri_t __pyx_f_4h5py_6_proxy_needs_proxy(hid_t); /*proto*/
#define __Pyx_MODULE_NAME "h5py._proxy"
int __pyx_module_is_main_h5py___proxy = 0;

/* Implementation of 'h5py._proxy' */
static PyObject *__pyx_builtin_xrange;
static PyObject *__pyx_builtin_MemoryError;
static PyObject *__pyx_builtin_RuntimeError;
static char __pyx_k_1[] = "Failed to allocate conversion buffer";
static char __pyx_k_3[] = "Illegal direction";
static char __pyx_k_5[] = "Failed to find converter for %s -> %s";
static char __pyx_k_6[] = "\n    Proxy functions for read/write, to work around the HDF5 bogus type issue.\n";
static char __pyx_k__range[] = "range";
static char __pyx_k__xrange[] = "xrange";
static char __pyx_k____main__[] = "__main__";
static char __pyx_k____test__[] = "__test__";
static char __pyx_k__MemoryError[] = "MemoryError";
static char __pyx_k__RuntimeError[] = "RuntimeError";
static PyObject *__pyx_kp_s_1;
static PyObject *__pyx_kp_s_3;
static PyObject *__pyx_kp_s_5;
static PyObject *__pyx_n_s__MemoryError;
static PyObject *__pyx_n_s__RuntimeError;
static PyObject *__pyx_n_s____main__;
static PyObject *__pyx_n_s____test__;
static PyObject *__pyx_n_s__range;
static PyObject *__pyx_n_s__xrange;
static PyObject *__pyx_k_tuple_2;
static PyObject *__pyx_k_tuple_4;

/* "h5py/_proxy.pyx":20
 *     H5PY_GATHER
 * 
 * cdef herr_t attr_rw(hid_t attr, hid_t mtype, void *progbuf, int read) except -1:             # <<<<<<<<<<<<<<
 * 
 *     cdef htri_t need_bkg
 */

static herr_t __pyx_f_4h5py_6_proxy_attr_rw(hid_t __pyx_v_attr, hid_t __pyx_v_mtype, void *__pyx_v_progbuf, int __pyx_v_read) {
  htri_t __pyx_v_need_bkg;
  hid_t __pyx_v_atype;
  hid_t __pyx_v_aspace;
  hsize_t __pyx_v_npoints;
  size_t __pyx_v_msize;
  size_t __pyx_v_asize;
  void *__pyx_v_conv_buf;
  void *__pyx_v_back_buf;
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  hid_t __pyx_t_1;
  htri_t __pyx_t_2;
  htri_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  herr_t __pyx_t_6;
  size_t __pyx_t_7;
  hssize_t __pyx_t_8;
  void *__pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("attr_rw", 0);

  /* "h5py/_proxy.pyx":23
 * 
 *     cdef htri_t need_bkg
 *     cdef hid_t atype = -1             # <<<<<<<<<<<<<<
 *     cdef hid_t aspace = -1
 *     cdef hsize_t npoints
 */
  __pyx_v_atype = -1;

  /* "h5py/_proxy.pyx":24
 *     cdef htri_t need_bkg
 *     cdef hid_t atype = -1
 *     cdef hid_t aspace = -1             # <<<<<<<<<<<<<<
 *     cdef hsize_t npoints
 * 
 */
  __pyx_v_aspace = -1;

  /* "h5py/_proxy.pyx":28
 * 
 *     cdef size_t msize, asize
 *     cdef void* conv_buf = NULL             # <<<<<<<<<<<<<<
 *     cdef void* back_buf = NULL
 * 
 */
  __pyx_v_conv_buf = NULL;

  /* "h5py/_proxy.pyx":29
 *     cdef size_t msize, asize
 *     cdef void* conv_buf = NULL
 *     cdef void* back_buf = NULL             # <<<<<<<<<<<<<<
 * 
 *     try:
 */
  __pyx_v_back_buf = NULL;

  /* "h5py/_proxy.pyx":31
 *     cdef void* back_buf = NULL
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         atype = H5Aget_type(attr)
 * 
 */
  /*try:*/ {

    /* "h5py/_proxy.pyx":32
 * 
 *     try:
 *         atype = H5Aget_type(attr)             # <<<<<<<<<<<<<<
 * 
 *         if not (needs_proxy(atype) or needs_proxy(mtype)):
 */
    __pyx_t_1 = __pyx_f_4h5py_4defs_H5Aget_type(__pyx_v_attr); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 32; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __pyx_v_atype = __pyx_t_1;

    /* "h5py/_proxy.pyx":34
 *         atype = H5Aget_type(attr)
 * 
 *         if not (needs_proxy(atype) or needs_proxy(mtype)):             # <<<<<<<<<<<<<<
 *             if read:
 *                 H5Aread(attr, mtype, progbuf)
 */
    __pyx_t_2 = __pyx_f_4h5py_6_proxy_needs_proxy(__pyx_v_atype); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 34; __pyx_clineno = __LINE__; goto __pyx_L4;}
    if (!__pyx_t_2) {
      __pyx_t_3 = __pyx_f_4h5py_6_proxy_needs_proxy(__pyx_v_mtype); if (unlikely(__pyx_t_3 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 34; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_t_4 = __pyx_t_3;
    } else {
      __pyx_t_4 = __pyx_t_2;
    }
    __pyx_t_5 = (!__pyx_t_4);
    if (__pyx_t_5) {

      /* "h5py/_proxy.pyx":35
 * 
 *         if not (needs_proxy(atype) or needs_proxy(mtype)):
 *             if read:             # <<<<<<<<<<<<<<
 *                 H5Aread(attr, mtype, progbuf)
 *             else:
 */
      if (__pyx_v_read) {

        /* "h5py/_proxy.pyx":36
 *         if not (needs_proxy(atype) or needs_proxy(mtype)):
 *             if read:
 *                 H5Aread(attr, mtype, progbuf)             # <<<<<<<<<<<<<<
 *             else:
 *                 H5Awrite(attr, mtype, progbuf)
 */
        __pyx_t_6 = __pyx_f_4h5py_4defs_H5Aread(__pyx_v_attr, __pyx_v_mtype, __pyx_v_progbuf); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 36; __pyx_clineno = __LINE__; goto __pyx_L4;}
        goto __pyx_L7;
      }
      /*else*/ {

        /* "h5py/_proxy.pyx":38
 *                 H5Aread(attr, mtype, progbuf)
 *             else:
 *                 H5Awrite(attr, mtype, progbuf)             # <<<<<<<<<<<<<<
 * 
 *         else:
 */
        __pyx_t_6 = __pyx_f_4h5py_4defs_H5Awrite(__pyx_v_attr, __pyx_v_mtype, __pyx_v_progbuf); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 38; __pyx_clineno = __LINE__; goto __pyx_L4;}
      }
      __pyx_L7:;
      goto __pyx_L6;
    }
    /*else*/ {

      /* "h5py/_proxy.pyx":42
 *         else:
 * 
 *             asize = H5Tget_size(atype)             # <<<<<<<<<<<<<<
 *             msize = H5Tget_size(mtype)
 *             aspace = H5Aget_space(attr)
 */
      __pyx_t_7 = __pyx_f_4h5py_4defs_H5Tget_size(__pyx_v_atype); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 42; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_v_asize = __pyx_t_7;

      /* "h5py/_proxy.pyx":43
 * 
 *             asize = H5Tget_size(atype)
 *             msize = H5Tget_size(mtype)             # <<<<<<<<<<<<<<
 *             aspace = H5Aget_space(attr)
 *             npoints = H5Sget_select_npoints(aspace)
 */
      __pyx_t_7 = __pyx_f_4h5py_4defs_H5Tget_size(__pyx_v_mtype); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 43; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_v_msize = __pyx_t_7;

      /* "h5py/_proxy.pyx":44
 *             asize = H5Tget_size(atype)
 *             msize = H5Tget_size(mtype)
 *             aspace = H5Aget_space(attr)             # <<<<<<<<<<<<<<
 *             npoints = H5Sget_select_npoints(aspace)
 * 
 */
      __pyx_t_1 = __pyx_f_4h5py_4defs_H5Aget_space(__pyx_v_attr); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 44; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_v_aspace = __pyx_t_1;

      /* "h5py/_proxy.pyx":45
 *             msize = H5Tget_size(mtype)
 *             aspace = H5Aget_space(attr)
 *             npoints = H5Sget_select_npoints(aspace)             # <<<<<<<<<<<<<<
 * 
 *             conv_buf = create_buffer(asize, msize, npoints)
 */
      __pyx_t_8 = __pyx_f_4h5py_4defs_H5Sget_select_npoints(__pyx_v_aspace); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 45; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_v_npoints = __pyx_t_8;

      /* "h5py/_proxy.pyx":47
 *             npoints = H5Sget_select_npoints(aspace)
 * 
 *             conv_buf = create_buffer(asize, msize, npoints)             # <<<<<<<<<<<<<<
 * 
 *             if read:
 */
      __pyx_t_9 = __pyx_f_4h5py_6_proxy_create_buffer(__pyx_v_asize, __pyx_v_msize, __pyx_v_npoints); if (unlikely(__pyx_t_9 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 47; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_v_conv_buf = __pyx_t_9;

      /* "h5py/_proxy.pyx":49
 *             conv_buf = create_buffer(asize, msize, npoints)
 * 
 *             if read:             # <<<<<<<<<<<<<<
 *                 need_bkg = needs_bkg_buffer(atype, mtype)
 *             else:
 */
      if (__pyx_v_read) {

        /* "h5py/_proxy.pyx":50
 * 
 *             if read:
 *                 need_bkg = needs_bkg_buffer(atype, mtype)             # <<<<<<<<<<<<<<
 *             else:
 *                 need_bkg = needs_bkg_buffer(mtype, atype)
 */
        __pyx_t_2 = __pyx_f_4h5py_6_proxy_needs_bkg_buffer(__pyx_v_atype, __pyx_v_mtype); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 50; __pyx_clineno = __LINE__; goto __pyx_L4;}
        __pyx_v_need_bkg = __pyx_t_2;
        goto __pyx_L8;
      }
      /*else*/ {

        /* "h5py/_proxy.pyx":52
 *                 need_bkg = needs_bkg_buffer(atype, mtype)
 *             else:
 *                 need_bkg = needs_bkg_buffer(mtype, atype)             # <<<<<<<<<<<<<<
 *             if need_bkg:
 *                 back_buf = malloc(msize*npoints)
 */
        __pyx_t_2 = __pyx_f_4h5py_6_proxy_needs_bkg_buffer(__pyx_v_mtype, __pyx_v_atype); if (unlikely(__pyx_t_2 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 52; __pyx_clineno = __LINE__; goto __pyx_L4;}
        __pyx_v_need_bkg = __pyx_t_2;
      }
      __pyx_L8:;

      /* "h5py/_proxy.pyx":53
 *             else:
 *                 need_bkg = needs_bkg_buffer(mtype, atype)
 *             if need_bkg:             # <<<<<<<<<<<<<<
 *                 back_buf = malloc(msize*npoints)
 *                 memcpy(back_buf, progbuf, msize*npoints)
 */
      if (__pyx_v_need_bkg) {

        /* "h5py/_proxy.pyx":54
 *                 need_bkg = needs_bkg_buffer(mtype, atype)
 *             if need_bkg:
 *                 back_buf = malloc(msize*npoints)             # <<<<<<<<<<<<<<
 *                 memcpy(back_buf, progbuf, msize*npoints)
 * 
 */
        __pyx_v_back_buf = malloc((__pyx_v_msize * __pyx_v_npoints));

        /* "h5py/_proxy.pyx":55
 *             if need_bkg:
 *                 back_buf = malloc(msize*npoints)
 *                 memcpy(back_buf, progbuf, msize*npoints)             # <<<<<<<<<<<<<<
 * 
 *             if read:
 */
        memcpy(__pyx_v_back_buf, __pyx_v_progbuf, (__pyx_v_msize * __pyx_v_npoints));
        goto __pyx_L9;
      }
      __pyx_L9:;

      /* "h5py/_proxy.pyx":57
 *                 memcpy(back_buf, progbuf, msize*npoints)
 * 
 *             if read:             # <<<<<<<<<<<<<<
 *                 H5Aread(attr, atype, conv_buf)
 *                 H5Tconvert(atype, mtype, npoints, conv_buf, back_buf, H5P_DEFAULT)
 */
      if (__pyx_v_read) {

        /* "h5py/_proxy.pyx":58
 * 
 *             if read:
 *                 H5Aread(attr, atype, conv_buf)             # <<<<<<<<<<<<<<
 *                 H5Tconvert(atype, mtype, npoints, conv_buf, back_buf, H5P_DEFAULT)
 *                 memcpy(progbuf, conv_buf, msize*npoints)
 */
        __pyx_t_6 = __pyx_f_4h5py_4defs_H5Aread(__pyx_v_attr, __pyx_v_atype, __pyx_v_conv_buf); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 58; __pyx_clineno = __LINE__; goto __pyx_L4;}

        /* "h5py/_proxy.pyx":59
 *             if read:
 *                 H5Aread(attr, atype, conv_buf)
 *                 H5Tconvert(atype, mtype, npoints, conv_buf, back_buf, H5P_DEFAULT)             # <<<<<<<<<<<<<<
 *                 memcpy(progbuf, conv_buf, msize*npoints)
 *             else:
 */
        __pyx_t_6 = __pyx_f_4h5py_4defs_H5Tconvert(__pyx_v_atype, __pyx_v_mtype, __pyx_v_npoints, __pyx_v_conv_buf, __pyx_v_back_buf, H5P_DEFAULT); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 59; __pyx_clineno = __LINE__; goto __pyx_L4;}

        /* "h5py/_proxy.pyx":60
 *                 H5Aread(attr, atype, conv_buf)
 *                 H5Tconvert(atype, mtype, npoints, conv_buf, back_buf, H5P_DEFAULT)
 *                 memcpy(progbuf, conv_buf, msize*npoints)             # <<<<<<<<<<<<<<
 *             else:
 *                 memcpy(conv_buf, progbuf, msize*npoints)
 */
        memcpy(__pyx_v_progbuf, __pyx_v_conv_buf, (__pyx_v_msize * __pyx_v_npoints));
        goto __pyx_L10;
      }
      /*else*/ {

        /* "h5py/_proxy.pyx":62
 *                 memcpy(progbuf, conv_buf, msize*npoints)
 *             else:
 *                 memcpy(conv_buf, progbuf, msize*npoints)             # <<<<<<<<<<<<<<
 *                 H5Tconvert(mtype, atype, npoints, conv_buf, back_buf, H5P_DEFAULT)
 *                 H5Awrite(attr, atype, conv_buf)
 */
        memcpy(__pyx_v_conv_buf, __pyx_v_progbuf, (__pyx_v_msize * __pyx_v_npoints));

        /* "h5py/_proxy.pyx":63
 *             else:
 *                 memcpy(conv_buf, progbuf, msize*npoints)
 *                 H5Tconvert(mtype, atype, npoints, conv_buf, back_buf, H5P_DEFAULT)             # <<<<<<<<<<<<<<
 *                 H5Awrite(attr, atype, conv_buf)
 *                 H5Dvlen_reclaim(atype, aspace, H5P_DEFAULT, conv_buf)
 */
        __pyx_t_6 = __pyx_f_4h5py_4defs_H5Tconvert(__pyx_v_mtype, __pyx_v_atype, __pyx_v_npoints, __pyx_v_conv_buf, __pyx_v_back_buf, H5P_DEFAULT); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 63; __pyx_clineno = __LINE__; goto __pyx_L4;}

        /* "h5py/_proxy.pyx":64
 *                 memcpy(conv_buf, progbuf, msize*npoints)
 *                 H5Tconvert(mtype, atype, npoints, conv_buf, back_buf, H5P_DEFAULT)
 *                 H5Awrite(attr, atype, conv_buf)             # <<<<<<<<<<<<<<
 *                 H5Dvlen_reclaim(atype, aspace, H5P_DEFAULT, conv_buf)
 * 
 */
        __pyx_t_6 = __pyx_f_4h5py_4defs_H5Awrite(__pyx_v_attr, __pyx_v_atype, __pyx_v_conv_buf); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 64; __pyx_clineno = __LINE__; goto __pyx_L4;}

        /* "h5py/_proxy.pyx":65
 *                 H5Tconvert(mtype, atype, npoints, conv_buf, back_buf, H5P_DEFAULT)
 *                 H5Awrite(attr, atype, conv_buf)
 *                 H5Dvlen_reclaim(atype, aspace, H5P_DEFAULT, conv_buf)             # <<<<<<<<<<<<<<
 * 
 *     finally:
 */
        __pyx_t_6 = __pyx_f_4h5py_4defs_H5Dvlen_reclaim(__pyx_v_atype, __pyx_v_aspace, H5P_DEFAULT, __pyx_v_conv_buf); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 65; __pyx_clineno = __LINE__; goto __pyx_L4;}
      }
      __pyx_L10:;
    }
    __pyx_L6:;
  }

  /* "h5py/_proxy.pyx":68
 * 
 *     finally:
 *         free(conv_buf)             # <<<<<<<<<<<<<<
 *         free(back_buf)
 *         if atype > 0:
 */
  /*finally:*/ {
    int __pyx_why;
    PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
    int __pyx_exc_lineno;
    __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 0; goto __pyx_L5;
    __pyx_L4: {
      __pyx_why = 4;
      __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
      __pyx_exc_lineno = __pyx_lineno;
      goto __pyx_L5;
    }
    __pyx_L5:;
    free(__pyx_v_conv_buf);

    /* "h5py/_proxy.pyx":69
 *     finally:
 *         free(conv_buf)
 *         free(back_buf)             # <<<<<<<<<<<<<<
 *         if atype > 0:
 *             H5Tclose(atype)
 */
    free(__pyx_v_back_buf);

    /* "h5py/_proxy.pyx":70
 *         free(conv_buf)
 *         free(back_buf)
 *         if atype > 0:             # <<<<<<<<<<<<<<
 *             H5Tclose(atype)
 *         if aspace > 0:
 */
    __pyx_t_5 = (__pyx_v_atype > 0);
    if (__pyx_t_5) {

      /* "h5py/_proxy.pyx":71
 *         free(back_buf)
 *         if atype > 0:
 *             H5Tclose(atype)             # <<<<<<<<<<<<<<
 *         if aspace > 0:
 *             H5Sclose(aspace)
 */
      __pyx_t_6 = __pyx_f_4h5py_4defs_H5Tclose(__pyx_v_atype); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 71; __pyx_clineno = __LINE__; goto __pyx_L11_error;}
      goto __pyx_L12;
    }
    __pyx_L12:;

    /* "h5py/_proxy.pyx":72
 *         if atype > 0:
 *             H5Tclose(atype)
 *         if aspace > 0:             # <<<<<<<<<<<<<<
 *             H5Sclose(aspace)
 * 
 */
    __pyx_t_5 = (__pyx_v_aspace > 0);
    if (__pyx_t_5) {

      /* "h5py/_proxy.pyx":73
 *             H5Tclose(atype)
 *         if aspace > 0:
 *             H5Sclose(aspace)             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
      __pyx_t_6 = __pyx_f_4h5py_4defs_H5Sclose(__pyx_v_aspace); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 73; __pyx_clineno = __LINE__; goto __pyx_L11_error;}
      goto __pyx_L13;
    }
    __pyx_L13:;
    goto __pyx_L14;
    __pyx_L11_error:;
    if (__pyx_why == 4) {
      Py_XDECREF(__pyx_exc_type);
      Py_XDECREF(__pyx_exc_value);
      Py_XDECREF(__pyx_exc_tb);
    }
    goto __pyx_L1_error;
    __pyx_L14:;
    switch (__pyx_why) {
      case 4: {
        __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
        __pyx_lineno = __pyx_exc_lineno;
        __pyx_exc_type = 0;
        __pyx_exc_value = 0;
        __pyx_exc_tb = 0;
        goto __pyx_L1_error;
      }
    }
  }

  /* "h5py/_proxy.pyx":75
 *             H5Sclose(aspace)
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * # =============================================================================
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._proxy.attr_rw", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_proxy.pyx":80
 * # Proxy functions to safely release the GIL around read/write operations
 * 
 * cdef herr_t H5PY_H5Dread(hid_t dset, hid_t mtype, hid_t mspace,             # <<<<<<<<<<<<<<
 *                         hid_t fspace, hid_t dxpl, void* buf) except -1:
 *     cdef herr_t retval
 */

static herr_t __pyx_f_4h5py_6_proxy_H5PY_H5Dread(hid_t __pyx_v_dset, hid_t __pyx_v_mtype, hid_t __pyx_v_mspace, hid_t __pyx_v_fspace, hid_t __pyx_v_dxpl, void *__pyx_v_buf) {
  herr_t __pyx_v_retval;
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("H5PY_H5Dread", 0);

  /* "h5py/_proxy.pyx":84
 *     cdef herr_t retval
 *     #with nogil:
 *     retval = H5Dread(dset, mtype, mspace, fspace, dxpl, buf)             # <<<<<<<<<<<<<<
 *     if retval < 0:
 *         return -1
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Dread(__pyx_v_dset, __pyx_v_mtype, __pyx_v_mspace, __pyx_v_fspace, __pyx_v_dxpl, __pyx_v_buf); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 84; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_retval = __pyx_t_1;

  /* "h5py/_proxy.pyx":85
 *     #with nogil:
 *     retval = H5Dread(dset, mtype, mspace, fspace, dxpl, buf)
 *     if retval < 0:             # <<<<<<<<<<<<<<
 *         return -1
 *     return retval
 */
  __pyx_t_2 = (__pyx_v_retval < 0);
  if (__pyx_t_2) {

    /* "h5py/_proxy.pyx":86
 *     retval = H5Dread(dset, mtype, mspace, fspace, dxpl, buf)
 *     if retval < 0:
 *         return -1             # <<<<<<<<<<<<<<
 *     return retval
 * 
 */
    __pyx_r = -1;
    goto __pyx_L0;
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/_proxy.pyx":87
 *     if retval < 0:
 *         return -1
 *     return retval             # <<<<<<<<<<<<<<
 * 
 * cdef herr_t H5PY_H5Dwrite(hid_t dset, hid_t mtype, hid_t mspace,
 */
  __pyx_r = __pyx_v_retval;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._proxy.H5PY_H5Dread", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_proxy.pyx":89
 *     return retval
 * 
 * cdef herr_t H5PY_H5Dwrite(hid_t dset, hid_t mtype, hid_t mspace,             # <<<<<<<<<<<<<<
 *                         hid_t fspace, hid_t dxpl, void* buf) except -1:
 *     cdef herr_t retval
 */

static herr_t __pyx_f_4h5py_6_proxy_H5PY_H5Dwrite(hid_t __pyx_v_dset, hid_t __pyx_v_mtype, hid_t __pyx_v_mspace, hid_t __pyx_v_fspace, hid_t __pyx_v_dxpl, void *__pyx_v_buf) {
  herr_t __pyx_v_retval;
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  herr_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("H5PY_H5Dwrite", 0);

  /* "h5py/_proxy.pyx":93
 *     cdef herr_t retval
 *     #with nogil:
 *     retval = H5Dwrite(dset, mtype, mspace, fspace, dxpl, buf)             # <<<<<<<<<<<<<<
 *     if retval < 0:
 *         return -1
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Dwrite(__pyx_v_dset, __pyx_v_mtype, __pyx_v_mspace, __pyx_v_fspace, __pyx_v_dxpl, __pyx_v_buf); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 93; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_retval = __pyx_t_1;

  /* "h5py/_proxy.pyx":94
 *     #with nogil:
 *     retval = H5Dwrite(dset, mtype, mspace, fspace, dxpl, buf)
 *     if retval < 0:             # <<<<<<<<<<<<<<
 *         return -1
 *     return retval
 */
  __pyx_t_2 = (__pyx_v_retval < 0);
  if (__pyx_t_2) {

    /* "h5py/_proxy.pyx":95
 *     retval = H5Dwrite(dset, mtype, mspace, fspace, dxpl, buf)
 *     if retval < 0:
 *         return -1             # <<<<<<<<<<<<<<
 *     return retval
 * 
 */
    __pyx_r = -1;
    goto __pyx_L0;
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/_proxy.pyx":96
 *     if retval < 0:
 *         return -1
 *     return retval             # <<<<<<<<<<<<<<
 * 
 * # =============================================================================
 */
  __pyx_r = __pyx_v_retval;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._proxy.H5PY_H5Dwrite", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_proxy.pyx":102
 * 
 * 
 * cdef herr_t dset_rw(hid_t dset, hid_t mtype, hid_t mspace, hid_t fspace,             # <<<<<<<<<<<<<<
 *                     hid_t dxpl, void* progbuf, int read) except -1:
 * 
 */

static herr_t __pyx_f_4h5py_6_proxy_dset_rw(hid_t __pyx_v_dset, hid_t __pyx_v_mtype, hid_t __pyx_v_mspace, hid_t __pyx_v_fspace, hid_t __pyx_v_dxpl, void *__pyx_v_progbuf, int __pyx_v_read) {
  htri_t __pyx_v_need_bkg;
  hid_t __pyx_v_dstype;
  hid_t __pyx_v_rawdstype;
  hid_t __pyx_v_dspace;
  hid_t __pyx_v_cspace;
  void *__pyx_v_back_buf;
  void *__pyx_v_conv_buf;
  hsize_t __pyx_v_npoints;
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  enum H5T_class_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  hid_t __pyx_t_5;
  herr_t __pyx_t_6;
  htri_t __pyx_t_7;
  htri_t __pyx_t_8;
  hssize_t __pyx_t_9;
  size_t __pyx_t_10;
  size_t __pyx_t_11;
  void *__pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("dset_rw", 0);

  /* "h5py/_proxy.pyx":106
 * 
 *     cdef htri_t need_bkg
 *     cdef hid_t dstype = -1      # Dataset datatype             # <<<<<<<<<<<<<<
 *     cdef hid_t rawdstype = -1
 *     cdef hid_t dspace = -1      # Dataset dataspace
 */
  __pyx_v_dstype = -1;

  /* "h5py/_proxy.pyx":107
 *     cdef htri_t need_bkg
 *     cdef hid_t dstype = -1      # Dataset datatype
 *     cdef hid_t rawdstype = -1             # <<<<<<<<<<<<<<
 *     cdef hid_t dspace = -1      # Dataset dataspace
 *     cdef hid_t cspace = -1      # Temporary contiguous dataspaces
 */
  __pyx_v_rawdstype = -1;

  /* "h5py/_proxy.pyx":108
 *     cdef hid_t dstype = -1      # Dataset datatype
 *     cdef hid_t rawdstype = -1
 *     cdef hid_t dspace = -1      # Dataset dataspace             # <<<<<<<<<<<<<<
 *     cdef hid_t cspace = -1      # Temporary contiguous dataspaces
 * 
 */
  __pyx_v_dspace = -1;

  /* "h5py/_proxy.pyx":109
 *     cdef hid_t rawdstype = -1
 *     cdef hid_t dspace = -1      # Dataset dataspace
 *     cdef hid_t cspace = -1      # Temporary contiguous dataspaces             # <<<<<<<<<<<<<<
 * 
 *     cdef void* back_buf = NULL
 */
  __pyx_v_cspace = -1;

  /* "h5py/_proxy.pyx":111
 *     cdef hid_t cspace = -1      # Temporary contiguous dataspaces
 * 
 *     cdef void* back_buf = NULL             # <<<<<<<<<<<<<<
 *     cdef void* conv_buf = NULL
 *     cdef hsize_t npoints
 */
  __pyx_v_back_buf = NULL;

  /* "h5py/_proxy.pyx":112
 * 
 *     cdef void* back_buf = NULL
 *     cdef void* conv_buf = NULL             # <<<<<<<<<<<<<<
 *     cdef hsize_t npoints
 * 
 */
  __pyx_v_conv_buf = NULL;

  /* "h5py/_proxy.pyx":115
 *     cdef hsize_t npoints
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         # Issue 372: when a compound type is involved, using the dataset type
 *         # may result in uninitialized data being sent to H5Tconvert for fields
 */
  /*try:*/ {

    /* "h5py/_proxy.pyx":121
 *         # to only those fields present in the memory type.  We can't use the
 *         # memory type directly because of course that triggers HDFFV-1063.
 *         if (H5Tget_class(mtype) == H5T_COMPOUND) and (not read):             # <<<<<<<<<<<<<<
 *             rawdstype = H5Dget_type(dset)
 *             dstype = make_reduced_type(mtype, rawdstype)
 */
    __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tget_class(__pyx_v_mtype); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 121; __pyx_clineno = __LINE__; goto __pyx_L4;}
    __pyx_t_2 = (__pyx_t_1 == H5T_COMPOUND);
    if (__pyx_t_2) {
      __pyx_t_3 = (!__pyx_v_read);
      __pyx_t_4 = __pyx_t_3;
    } else {
      __pyx_t_4 = __pyx_t_2;
    }
    if (__pyx_t_4) {

      /* "h5py/_proxy.pyx":122
 *         # memory type directly because of course that triggers HDFFV-1063.
 *         if (H5Tget_class(mtype) == H5T_COMPOUND) and (not read):
 *             rawdstype = H5Dget_type(dset)             # <<<<<<<<<<<<<<
 *             dstype = make_reduced_type(mtype, rawdstype)
 *             H5Tclose(rawdstype)
 */
      __pyx_t_5 = __pyx_f_4h5py_4defs_H5Dget_type(__pyx_v_dset); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 122; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_v_rawdstype = __pyx_t_5;

      /* "h5py/_proxy.pyx":123
 *         if (H5Tget_class(mtype) == H5T_COMPOUND) and (not read):
 *             rawdstype = H5Dget_type(dset)
 *             dstype = make_reduced_type(mtype, rawdstype)             # <<<<<<<<<<<<<<
 *             H5Tclose(rawdstype)
 *         else:
 */
      __pyx_v_dstype = __pyx_f_4h5py_6_proxy_make_reduced_type(__pyx_v_mtype, __pyx_v_rawdstype);

      /* "h5py/_proxy.pyx":124
 *             rawdstype = H5Dget_type(dset)
 *             dstype = make_reduced_type(mtype, rawdstype)
 *             H5Tclose(rawdstype)             # <<<<<<<<<<<<<<
 *         else:
 *             dstype = H5Dget_type(dset)
 */
      __pyx_t_6 = __pyx_f_4h5py_4defs_H5Tclose(__pyx_v_rawdstype); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 124; __pyx_clineno = __LINE__; goto __pyx_L4;}
      goto __pyx_L6;
    }
    /*else*/ {

      /* "h5py/_proxy.pyx":126
 *             H5Tclose(rawdstype)
 *         else:
 *             dstype = H5Dget_type(dset)             # <<<<<<<<<<<<<<
 * 
 *         if not (needs_proxy(dstype) or needs_proxy(mtype)):
 */
      __pyx_t_5 = __pyx_f_4h5py_4defs_H5Dget_type(__pyx_v_dset); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 126; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_v_dstype = __pyx_t_5;
    }
    __pyx_L6:;

    /* "h5py/_proxy.pyx":128
 *             dstype = H5Dget_type(dset)
 * 
 *         if not (needs_proxy(dstype) or needs_proxy(mtype)):             # <<<<<<<<<<<<<<
 *             if read:
 *                 H5PY_H5Dread(dset, mtype, mspace, fspace, dxpl, progbuf)
 */
    __pyx_t_7 = __pyx_f_4h5py_6_proxy_needs_proxy(__pyx_v_dstype); if (unlikely(__pyx_t_7 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 128; __pyx_clineno = __LINE__; goto __pyx_L4;}
    if (!__pyx_t_7) {
      __pyx_t_8 = __pyx_f_4h5py_6_proxy_needs_proxy(__pyx_v_mtype); if (unlikely(__pyx_t_8 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 128; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_t_4 = __pyx_t_8;
    } else {
      __pyx_t_4 = __pyx_t_7;
    }
    __pyx_t_2 = (!__pyx_t_4);
    if (__pyx_t_2) {

      /* "h5py/_proxy.pyx":129
 * 
 *         if not (needs_proxy(dstype) or needs_proxy(mtype)):
 *             if read:             # <<<<<<<<<<<<<<
 *                 H5PY_H5Dread(dset, mtype, mspace, fspace, dxpl, progbuf)
 *             else:
 */
      if (__pyx_v_read) {

        /* "h5py/_proxy.pyx":130
 *         if not (needs_proxy(dstype) or needs_proxy(mtype)):
 *             if read:
 *                 H5PY_H5Dread(dset, mtype, mspace, fspace, dxpl, progbuf)             # <<<<<<<<<<<<<<
 *             else:
 *                 H5PY_H5Dwrite(dset, mtype, mspace, fspace, dxpl, progbuf)
 */
        __pyx_t_6 = __pyx_f_4h5py_6_proxy_H5PY_H5Dread(__pyx_v_dset, __pyx_v_mtype, __pyx_v_mspace, __pyx_v_fspace, __pyx_v_dxpl, __pyx_v_progbuf); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 130; __pyx_clineno = __LINE__; goto __pyx_L4;}
        goto __pyx_L8;
      }
      /*else*/ {

        /* "h5py/_proxy.pyx":132
 *                 H5PY_H5Dread(dset, mtype, mspace, fspace, dxpl, progbuf)
 *             else:
 *                 H5PY_H5Dwrite(dset, mtype, mspace, fspace, dxpl, progbuf)             # <<<<<<<<<<<<<<
 *         else:
 * 
 */
        __pyx_t_6 = __pyx_f_4h5py_6_proxy_H5PY_H5Dwrite(__pyx_v_dset, __pyx_v_mtype, __pyx_v_mspace, __pyx_v_fspace, __pyx_v_dxpl, __pyx_v_progbuf); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 132; __pyx_clineno = __LINE__; goto __pyx_L4;}
      }
      __pyx_L8:;
      goto __pyx_L7;
    }
    /*else*/ {

      /* "h5py/_proxy.pyx":135
 *         else:
 * 
 *             if mspace == H5S_ALL and fspace != H5S_ALL:             # <<<<<<<<<<<<<<
 *                 mspace = fspace
 *             elif mspace != H5S_ALL and fspace == H5S_ALL:
 */
      __pyx_t_2 = (__pyx_v_mspace == H5S_ALL);
      if (__pyx_t_2) {
        __pyx_t_4 = (__pyx_v_fspace != H5S_ALL);
        __pyx_t_3 = __pyx_t_4;
      } else {
        __pyx_t_3 = __pyx_t_2;
      }
      if (__pyx_t_3) {

        /* "h5py/_proxy.pyx":136
 * 
 *             if mspace == H5S_ALL and fspace != H5S_ALL:
 *                 mspace = fspace             # <<<<<<<<<<<<<<
 *             elif mspace != H5S_ALL and fspace == H5S_ALL:
 *                 fspace = mspace
 */
        __pyx_v_mspace = __pyx_v_fspace;
        goto __pyx_L9;
      }

      /* "h5py/_proxy.pyx":137
 *             if mspace == H5S_ALL and fspace != H5S_ALL:
 *                 mspace = fspace
 *             elif mspace != H5S_ALL and fspace == H5S_ALL:             # <<<<<<<<<<<<<<
 *                 fspace = mspace
 *             elif mspace == H5S_ALL and fspace == H5S_ALL:
 */
      __pyx_t_3 = (__pyx_v_mspace != H5S_ALL);
      if (__pyx_t_3) {
        __pyx_t_2 = (__pyx_v_fspace == H5S_ALL);
        __pyx_t_4 = __pyx_t_2;
      } else {
        __pyx_t_4 = __pyx_t_3;
      }
      if (__pyx_t_4) {

        /* "h5py/_proxy.pyx":138
 *                 mspace = fspace
 *             elif mspace != H5S_ALL and fspace == H5S_ALL:
 *                 fspace = mspace             # <<<<<<<<<<<<<<
 *             elif mspace == H5S_ALL and fspace == H5S_ALL:
 *                 fspace = mspace = dspace = H5Dget_space(dset)
 */
        __pyx_v_fspace = __pyx_v_mspace;
        goto __pyx_L9;
      }

      /* "h5py/_proxy.pyx":139
 *             elif mspace != H5S_ALL and fspace == H5S_ALL:
 *                 fspace = mspace
 *             elif mspace == H5S_ALL and fspace == H5S_ALL:             # <<<<<<<<<<<<<<
 *                 fspace = mspace = dspace = H5Dget_space(dset)
 * 
 */
      __pyx_t_4 = (__pyx_v_mspace == H5S_ALL);
      if (__pyx_t_4) {
        __pyx_t_3 = (__pyx_v_fspace == H5S_ALL);
        __pyx_t_2 = __pyx_t_3;
      } else {
        __pyx_t_2 = __pyx_t_4;
      }
      if (__pyx_t_2) {

        /* "h5py/_proxy.pyx":140
 *                 fspace = mspace
 *             elif mspace == H5S_ALL and fspace == H5S_ALL:
 *                 fspace = mspace = dspace = H5Dget_space(dset)             # <<<<<<<<<<<<<<
 * 
 *             npoints = H5Sget_select_npoints(mspace)
 */
        __pyx_t_5 = __pyx_f_4h5py_4defs_H5Dget_space(__pyx_v_dset); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 140; __pyx_clineno = __LINE__; goto __pyx_L4;}
        __pyx_v_fspace = __pyx_t_5;
        __pyx_v_mspace = __pyx_t_5;
        __pyx_v_dspace = __pyx_t_5;
        goto __pyx_L9;
      }
      __pyx_L9:;

      /* "h5py/_proxy.pyx":142
 *                 fspace = mspace = dspace = H5Dget_space(dset)
 * 
 *             npoints = H5Sget_select_npoints(mspace)             # <<<<<<<<<<<<<<
 *             cspace = H5Screate_simple(1, &npoints, NULL)
 * 
 */
      __pyx_t_9 = __pyx_f_4h5py_4defs_H5Sget_select_npoints(__pyx_v_mspace); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 142; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_v_npoints = __pyx_t_9;

      /* "h5py/_proxy.pyx":143
 * 
 *             npoints = H5Sget_select_npoints(mspace)
 *             cspace = H5Screate_simple(1, &npoints, NULL)             # <<<<<<<<<<<<<<
 * 
 *             conv_buf = create_buffer(H5Tget_size(dstype), H5Tget_size(mtype), npoints)
 */
      __pyx_t_5 = __pyx_f_4h5py_4defs_H5Screate_simple(1, (&__pyx_v_npoints), NULL); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 143; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_v_cspace = __pyx_t_5;

      /* "h5py/_proxy.pyx":145
 *             cspace = H5Screate_simple(1, &npoints, NULL)
 * 
 *             conv_buf = create_buffer(H5Tget_size(dstype), H5Tget_size(mtype), npoints)             # <<<<<<<<<<<<<<
 * 
 *             # Only create a (contiguous) backing buffer if absolutely
 */
      __pyx_t_10 = __pyx_f_4h5py_4defs_H5Tget_size(__pyx_v_dstype); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 145; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_t_11 = __pyx_f_4h5py_4defs_H5Tget_size(__pyx_v_mtype); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 145; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_t_12 = __pyx_f_4h5py_6_proxy_create_buffer(__pyx_t_10, __pyx_t_11, __pyx_v_npoints); if (unlikely(__pyx_t_12 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 145; __pyx_clineno = __LINE__; goto __pyx_L4;}
      __pyx_v_conv_buf = __pyx_t_12;

      /* "h5py/_proxy.pyx":149
 *             # Only create a (contiguous) backing buffer if absolutely
 *             # necessary. Note this buffer always has memory type.
 *             if read:             # <<<<<<<<<<<<<<
 *                 need_bkg = needs_bkg_buffer(dstype, mtype)
 *             else:
 */
      if (__pyx_v_read) {

        /* "h5py/_proxy.pyx":150
 *             # necessary. Note this buffer always has memory type.
 *             if read:
 *                 need_bkg = needs_bkg_buffer(dstype, mtype)             # <<<<<<<<<<<<<<
 *             else:
 *                 need_bkg = needs_bkg_buffer(mtype, dstype)
 */
        __pyx_t_7 = __pyx_f_4h5py_6_proxy_needs_bkg_buffer(__pyx_v_dstype, __pyx_v_mtype); if (unlikely(__pyx_t_7 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 150; __pyx_clineno = __LINE__; goto __pyx_L4;}
        __pyx_v_need_bkg = __pyx_t_7;
        goto __pyx_L10;
      }
      /*else*/ {

        /* "h5py/_proxy.pyx":152
 *                 need_bkg = needs_bkg_buffer(dstype, mtype)
 *             else:
 *                 need_bkg = needs_bkg_buffer(mtype, dstype)             # <<<<<<<<<<<<<<
 *             if need_bkg:
 *                 back_buf = create_buffer(H5Tget_size(dstype), H5Tget_size(mtype), npoints)
 */
        __pyx_t_7 = __pyx_f_4h5py_6_proxy_needs_bkg_buffer(__pyx_v_mtype, __pyx_v_dstype); if (unlikely(__pyx_t_7 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 152; __pyx_clineno = __LINE__; goto __pyx_L4;}
        __pyx_v_need_bkg = __pyx_t_7;
      }
      __pyx_L10:;

      /* "h5py/_proxy.pyx":153
 *             else:
 *                 need_bkg = needs_bkg_buffer(mtype, dstype)
 *             if need_bkg:             # <<<<<<<<<<<<<<
 *                 back_buf = create_buffer(H5Tget_size(dstype), H5Tget_size(mtype), npoints)
 *                 h5py_copy(mtype, mspace, back_buf, progbuf, H5PY_GATHER)
 */
      if (__pyx_v_need_bkg) {

        /* "h5py/_proxy.pyx":154
 *                 need_bkg = needs_bkg_buffer(mtype, dstype)
 *             if need_bkg:
 *                 back_buf = create_buffer(H5Tget_size(dstype), H5Tget_size(mtype), npoints)             # <<<<<<<<<<<<<<
 *                 h5py_copy(mtype, mspace, back_buf, progbuf, H5PY_GATHER)
 * 
 */
        __pyx_t_11 = __pyx_f_4h5py_4defs_H5Tget_size(__pyx_v_dstype); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L4;}
        __pyx_t_10 = __pyx_f_4h5py_4defs_H5Tget_size(__pyx_v_mtype); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L4;}
        __pyx_t_12 = __pyx_f_4h5py_6_proxy_create_buffer(__pyx_t_11, __pyx_t_10, __pyx_v_npoints); if (unlikely(__pyx_t_12 == NULL)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 154; __pyx_clineno = __LINE__; goto __pyx_L4;}
        __pyx_v_back_buf = __pyx_t_12;

        /* "h5py/_proxy.pyx":155
 *             if need_bkg:
 *                 back_buf = create_buffer(H5Tget_size(dstype), H5Tget_size(mtype), npoints)
 *                 h5py_copy(mtype, mspace, back_buf, progbuf, H5PY_GATHER)             # <<<<<<<<<<<<<<
 * 
 *             if read:
 */
        __pyx_t_6 = __pyx_f_4h5py_6_proxy_h5py_copy(__pyx_v_mtype, __pyx_v_mspace, __pyx_v_back_buf, __pyx_v_progbuf, __pyx_e_4h5py_6_proxy_H5PY_GATHER); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 155; __pyx_clineno = __LINE__; goto __pyx_L4;}
        goto __pyx_L11;
      }
      __pyx_L11:;

      /* "h5py/_proxy.pyx":157
 *                 h5py_copy(mtype, mspace, back_buf, progbuf, H5PY_GATHER)
 * 
 *             if read:             # <<<<<<<<<<<<<<
 *                 H5PY_H5Dread(dset, dstype, cspace, fspace, dxpl, conv_buf)
 *                 H5Tconvert(dstype, mtype, npoints, conv_buf, back_buf, dxpl)
 */
      if (__pyx_v_read) {

        /* "h5py/_proxy.pyx":158
 * 
 *             if read:
 *                 H5PY_H5Dread(dset, dstype, cspace, fspace, dxpl, conv_buf)             # <<<<<<<<<<<<<<
 *                 H5Tconvert(dstype, mtype, npoints, conv_buf, back_buf, dxpl)
 *                 h5py_copy(mtype, mspace, conv_buf, progbuf, H5PY_SCATTER)
 */
        __pyx_t_6 = __pyx_f_4h5py_6_proxy_H5PY_H5Dread(__pyx_v_dset, __pyx_v_dstype, __pyx_v_cspace, __pyx_v_fspace, __pyx_v_dxpl, __pyx_v_conv_buf); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 158; __pyx_clineno = __LINE__; goto __pyx_L4;}

        /* "h5py/_proxy.pyx":159
 *             if read:
 *                 H5PY_H5Dread(dset, dstype, cspace, fspace, dxpl, conv_buf)
 *                 H5Tconvert(dstype, mtype, npoints, conv_buf, back_buf, dxpl)             # <<<<<<<<<<<<<<
 *                 h5py_copy(mtype, mspace, conv_buf, progbuf, H5PY_SCATTER)
 *             else:
 */
        __pyx_t_6 = __pyx_f_4h5py_4defs_H5Tconvert(__pyx_v_dstype, __pyx_v_mtype, __pyx_v_npoints, __pyx_v_conv_buf, __pyx_v_back_buf, __pyx_v_dxpl); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 159; __pyx_clineno = __LINE__; goto __pyx_L4;}

        /* "h5py/_proxy.pyx":160
 *                 H5PY_H5Dread(dset, dstype, cspace, fspace, dxpl, conv_buf)
 *                 H5Tconvert(dstype, mtype, npoints, conv_buf, back_buf, dxpl)
 *                 h5py_copy(mtype, mspace, conv_buf, progbuf, H5PY_SCATTER)             # <<<<<<<<<<<<<<
 *             else:
 *                 h5py_copy(mtype, mspace, conv_buf, progbuf, H5PY_GATHER)
 */
        __pyx_t_6 = __pyx_f_4h5py_6_proxy_h5py_copy(__pyx_v_mtype, __pyx_v_mspace, __pyx_v_conv_buf, __pyx_v_progbuf, __pyx_e_4h5py_6_proxy_H5PY_SCATTER); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 160; __pyx_clineno = __LINE__; goto __pyx_L4;}
        goto __pyx_L12;
      }
      /*else*/ {

        /* "h5py/_proxy.pyx":162
 *                 h5py_copy(mtype, mspace, conv_buf, progbuf, H5PY_SCATTER)
 *             else:
 *                 h5py_copy(mtype, mspace, conv_buf, progbuf, H5PY_GATHER)             # <<<<<<<<<<<<<<
 *                 H5Tconvert(mtype, dstype, npoints, conv_buf, back_buf, dxpl)
 *                 H5PY_H5Dwrite(dset, dstype, cspace, fspace, dxpl, conv_buf)
 */
        __pyx_t_6 = __pyx_f_4h5py_6_proxy_h5py_copy(__pyx_v_mtype, __pyx_v_mspace, __pyx_v_conv_buf, __pyx_v_progbuf, __pyx_e_4h5py_6_proxy_H5PY_GATHER); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 162; __pyx_clineno = __LINE__; goto __pyx_L4;}

        /* "h5py/_proxy.pyx":163
 *             else:
 *                 h5py_copy(mtype, mspace, conv_buf, progbuf, H5PY_GATHER)
 *                 H5Tconvert(mtype, dstype, npoints, conv_buf, back_buf, dxpl)             # <<<<<<<<<<<<<<
 *                 H5PY_H5Dwrite(dset, dstype, cspace, fspace, dxpl, conv_buf)
 *                 H5Dvlen_reclaim(dstype, cspace, H5P_DEFAULT, conv_buf)
 */
        __pyx_t_6 = __pyx_f_4h5py_4defs_H5Tconvert(__pyx_v_mtype, __pyx_v_dstype, __pyx_v_npoints, __pyx_v_conv_buf, __pyx_v_back_buf, __pyx_v_dxpl); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 163; __pyx_clineno = __LINE__; goto __pyx_L4;}

        /* "h5py/_proxy.pyx":164
 *                 h5py_copy(mtype, mspace, conv_buf, progbuf, H5PY_GATHER)
 *                 H5Tconvert(mtype, dstype, npoints, conv_buf, back_buf, dxpl)
 *                 H5PY_H5Dwrite(dset, dstype, cspace, fspace, dxpl, conv_buf)             # <<<<<<<<<<<<<<
 *                 H5Dvlen_reclaim(dstype, cspace, H5P_DEFAULT, conv_buf)
 * 
 */
        __pyx_t_6 = __pyx_f_4h5py_6_proxy_H5PY_H5Dwrite(__pyx_v_dset, __pyx_v_dstype, __pyx_v_cspace, __pyx_v_fspace, __pyx_v_dxpl, __pyx_v_conv_buf); if (unlikely(__pyx_t_6 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 164; __pyx_clineno = __LINE__; goto __pyx_L4;}

        /* "h5py/_proxy.pyx":165
 *                 H5Tconvert(mtype, dstype, npoints, conv_buf, back_buf, dxpl)
 *                 H5PY_H5Dwrite(dset, dstype, cspace, fspace, dxpl, conv_buf)
 *                 H5Dvlen_reclaim(dstype, cspace, H5P_DEFAULT, conv_buf)             # <<<<<<<<<<<<<<
 * 
 *     finally:
 */
        __pyx_t_6 = __pyx_f_4h5py_4defs_H5Dvlen_reclaim(__pyx_v_dstype, __pyx_v_cspace, H5P_DEFAULT, __pyx_v_conv_buf); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 165; __pyx_clineno = __LINE__; goto __pyx_L4;}
      }
      __pyx_L12:;
    }
    __pyx_L7:;
  }

  /* "h5py/_proxy.pyx":168
 * 
 *     finally:
 *         free(back_buf)             # <<<<<<<<<<<<<<
 *         free(conv_buf)
 *         if dstype > 0:
 */
  /*finally:*/ {
    int __pyx_why;
    PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
    int __pyx_exc_lineno;
    __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
    __pyx_why = 0; goto __pyx_L5;
    __pyx_L4: {
      __pyx_why = 4;
      __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
      __pyx_exc_lineno = __pyx_lineno;
      goto __pyx_L5;
    }
    __pyx_L5:;
    free(__pyx_v_back_buf);

    /* "h5py/_proxy.pyx":169
 *     finally:
 *         free(back_buf)
 *         free(conv_buf)             # <<<<<<<<<<<<<<
 *         if dstype > 0:
 *             H5Tclose(dstype)
 */
    free(__pyx_v_conv_buf);

    /* "h5py/_proxy.pyx":170
 *         free(back_buf)
 *         free(conv_buf)
 *         if dstype > 0:             # <<<<<<<<<<<<<<
 *             H5Tclose(dstype)
 *         if dspace > 0:
 */
    __pyx_t_2 = (__pyx_v_dstype > 0);
    if (__pyx_t_2) {

      /* "h5py/_proxy.pyx":171
 *         free(conv_buf)
 *         if dstype > 0:
 *             H5Tclose(dstype)             # <<<<<<<<<<<<<<
 *         if dspace > 0:
 *             H5Sclose(dspace)
 */
      __pyx_t_6 = __pyx_f_4h5py_4defs_H5Tclose(__pyx_v_dstype); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 171; __pyx_clineno = __LINE__; goto __pyx_L13_error;}
      goto __pyx_L14;
    }
    __pyx_L14:;

    /* "h5py/_proxy.pyx":172
 *         if dstype > 0:
 *             H5Tclose(dstype)
 *         if dspace > 0:             # <<<<<<<<<<<<<<
 *             H5Sclose(dspace)
 *         if cspace > 0:
 */
    __pyx_t_2 = (__pyx_v_dspace > 0);
    if (__pyx_t_2) {

      /* "h5py/_proxy.pyx":173
 *             H5Tclose(dstype)
 *         if dspace > 0:
 *             H5Sclose(dspace)             # <<<<<<<<<<<<<<
 *         if cspace > 0:
 *             H5Sclose(cspace)
 */
      __pyx_t_6 = __pyx_f_4h5py_4defs_H5Sclose(__pyx_v_dspace); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 173; __pyx_clineno = __LINE__; goto __pyx_L13_error;}
      goto __pyx_L15;
    }
    __pyx_L15:;

    /* "h5py/_proxy.pyx":174
 *         if dspace > 0:
 *             H5Sclose(dspace)
 *         if cspace > 0:             # <<<<<<<<<<<<<<
 *             H5Sclose(cspace)
 * 
 */
    __pyx_t_2 = (__pyx_v_cspace > 0);
    if (__pyx_t_2) {

      /* "h5py/_proxy.pyx":175
 *             H5Sclose(dspace)
 *         if cspace > 0:
 *             H5Sclose(cspace)             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
      __pyx_t_6 = __pyx_f_4h5py_4defs_H5Sclose(__pyx_v_cspace); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 175; __pyx_clineno = __LINE__; goto __pyx_L13_error;}
      goto __pyx_L16;
    }
    __pyx_L16:;
    goto __pyx_L17;
    __pyx_L13_error:;
    if (__pyx_why == 4) {
      Py_XDECREF(__pyx_exc_type);
      Py_XDECREF(__pyx_exc_value);
      Py_XDECREF(__pyx_exc_tb);
    }
    goto __pyx_L1_error;
    __pyx_L17:;
    switch (__pyx_why) {
      case 4: {
        __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
        __pyx_lineno = __pyx_exc_lineno;
        __pyx_exc_type = 0;
        __pyx_exc_value = 0;
        __pyx_exc_tb = 0;
        goto __pyx_L1_error;
      }
    }
  }

  /* "h5py/_proxy.pyx":177
 *             H5Sclose(cspace)
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._proxy.dset_rw", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_proxy.pyx":180
 * 
 * 
 * cdef hid_t make_reduced_type(hid_t mtype, hid_t dstype):             # <<<<<<<<<<<<<<
 *     # Go through dstype, pick out the fields which also appear in mtype, and
 *     # return a new compound type with the fields packed together
 */

static hid_t __pyx_f_4h5py_6_proxy_make_reduced_type(hid_t __pyx_v_mtype, hid_t __pyx_v_dstype) {
  hid_t __pyx_v_newtype;
  hid_t __pyx_v_temptype;
  hsize_t __pyx_v_newtype_size;
  hsize_t __pyx_v_offset;
  PyObject *__pyx_v_mtype_fields = NULL;
  int __pyx_v_idx;
  char *__pyx_v_name;
  hid_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  char *__pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  hid_t __pyx_t_7;
  size_t __pyx_t_8;
  herr_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("make_reduced_type", 0);

  /* "h5py/_proxy.pyx":189
 * 
 *     # Make a list of all names in the memory type.
 *     mtype_fields = []             # <<<<<<<<<<<<<<
 *     for idx in xrange(H5Tget_nmembers(mtype)):
 *         mtype_fields.append(H5Tget_member_name(mtype, idx))
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 189; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_mtype_fields = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "h5py/_proxy.pyx":190
 *     # Make a list of all names in the memory type.
 *     mtype_fields = []
 *     for idx in xrange(H5Tget_nmembers(mtype)):             # <<<<<<<<<<<<<<
 *         mtype_fields.append(H5Tget_member_name(mtype, idx))
 * 
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tget_nmembers(__pyx_v_mtype); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 190; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_idx = __pyx_t_3;

    /* "h5py/_proxy.pyx":191
 *     mtype_fields = []
 *     for idx in xrange(H5Tget_nmembers(mtype)):
 *         mtype_fields.append(H5Tget_member_name(mtype, idx))             # <<<<<<<<<<<<<<
 * 
 *     # First pass: add up the sizes of matching fields so we know how large a
 */
    __pyx_t_4 = __pyx_f_4h5py_4defs_H5Tget_member_name(__pyx_v_mtype, __pyx_v_idx); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_1 = PyBytes_FromString(__pyx_t_4); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(((PyObject *)__pyx_t_1));
    __pyx_t_5 = PyList_Append(__pyx_v_mtype_fields, ((PyObject *)__pyx_t_1)); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 191; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(((PyObject *)__pyx_t_1)); __pyx_t_1 = 0;
  }

  /* "h5py/_proxy.pyx":195
 *     # First pass: add up the sizes of matching fields so we know how large a
 *     # type to make
 *     newtype_size = 0             # <<<<<<<<<<<<<<
 *     for idx in xrange(H5Tget_nmembers(dstype)):
 *         name = H5Tget_member_name(dstype, idx)
 */
  __pyx_v_newtype_size = 0;

  /* "h5py/_proxy.pyx":196
 *     # type to make
 *     newtype_size = 0
 *     for idx in xrange(H5Tget_nmembers(dstype)):             # <<<<<<<<<<<<<<
 *         name = H5Tget_member_name(dstype, idx)
 *         if name not in mtype_fields:
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tget_nmembers(__pyx_v_dstype); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 196; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_idx = __pyx_t_3;

    /* "h5py/_proxy.pyx":197
 *     newtype_size = 0
 *     for idx in xrange(H5Tget_nmembers(dstype)):
 *         name = H5Tget_member_name(dstype, idx)             # <<<<<<<<<<<<<<
 *         if name not in mtype_fields:
 *             continue
 */
    __pyx_t_4 = __pyx_f_4h5py_4defs_H5Tget_member_name(__pyx_v_dstype, __pyx_v_idx); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 197; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_name = __pyx_t_4;

    /* "h5py/_proxy.pyx":198
 *     for idx in xrange(H5Tget_nmembers(dstype)):
 *         name = H5Tget_member_name(dstype, idx)
 *         if name not in mtype_fields:             # <<<<<<<<<<<<<<
 *             continue
 *         temptype = H5Tget_member_type(dstype, idx)
 */
    __pyx_t_1 = PyBytes_FromString(__pyx_v_name); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 198; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(((PyObject *)__pyx_t_1));
    __pyx_t_6 = (__Pyx_PySequence_Contains(((PyObject *)__pyx_t_1), ((PyObject *)__pyx_v_mtype_fields), Py_NE)); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 198; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(((PyObject *)__pyx_t_1)); __pyx_t_1 = 0;
    if (__pyx_t_6) {

      /* "h5py/_proxy.pyx":199
 *         name = H5Tget_member_name(dstype, idx)
 *         if name not in mtype_fields:
 *             continue             # <<<<<<<<<<<<<<
 *         temptype = H5Tget_member_type(dstype, idx)
 *         newtype_size += H5Tget_size(temptype)
 */
      goto __pyx_L5_continue;
      goto __pyx_L7;
    }
    __pyx_L7:;

    /* "h5py/_proxy.pyx":200
 *         if name not in mtype_fields:
 *             continue
 *         temptype = H5Tget_member_type(dstype, idx)             # <<<<<<<<<<<<<<
 *         newtype_size += H5Tget_size(temptype)
 *         H5Tclose(temptype)
 */
    __pyx_t_7 = __pyx_f_4h5py_4defs_H5Tget_member_type(__pyx_v_dstype, __pyx_v_idx); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 200; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_temptype = __pyx_t_7;

    /* "h5py/_proxy.pyx":201
 *             continue
 *         temptype = H5Tget_member_type(dstype, idx)
 *         newtype_size += H5Tget_size(temptype)             # <<<<<<<<<<<<<<
 *         H5Tclose(temptype)
 * 
 */
    __pyx_t_8 = __pyx_f_4h5py_4defs_H5Tget_size(__pyx_v_temptype); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 201; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_newtype_size = (__pyx_v_newtype_size + __pyx_t_8);

    /* "h5py/_proxy.pyx":202
 *         temptype = H5Tget_member_type(dstype, idx)
 *         newtype_size += H5Tget_size(temptype)
 *         H5Tclose(temptype)             # <<<<<<<<<<<<<<
 * 
 *     newtype = H5Tcreate(H5T_COMPOUND, newtype_size)
 */
    __pyx_t_9 = __pyx_f_4h5py_4defs_H5Tclose(__pyx_v_temptype); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 202; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_L5_continue:;
  }

  /* "h5py/_proxy.pyx":204
 *         H5Tclose(temptype)
 * 
 *     newtype = H5Tcreate(H5T_COMPOUND, newtype_size)             # <<<<<<<<<<<<<<
 * 
 *     # Second pass: pick out the matching fields and pack them in the new type
 */
  __pyx_t_7 = __pyx_f_4h5py_4defs_H5Tcreate(H5T_COMPOUND, __pyx_v_newtype_size); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 204; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_newtype = __pyx_t_7;

  /* "h5py/_proxy.pyx":207
 * 
 *     # Second pass: pick out the matching fields and pack them in the new type
 *     offset = 0             # <<<<<<<<<<<<<<
 *     for idx in xrange(H5Tget_nmembers(dstype)):
 *         name = H5Tget_member_name(dstype, idx)
 */
  __pyx_v_offset = 0;

  /* "h5py/_proxy.pyx":208
 *     # Second pass: pick out the matching fields and pack them in the new type
 *     offset = 0
 *     for idx in xrange(H5Tget_nmembers(dstype)):             # <<<<<<<<<<<<<<
 *         name = H5Tget_member_name(dstype, idx)
 *         if name not in mtype_fields:
 */
  __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tget_nmembers(__pyx_v_dstype); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 208; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_idx = __pyx_t_3;

    /* "h5py/_proxy.pyx":209
 *     offset = 0
 *     for idx in xrange(H5Tget_nmembers(dstype)):
 *         name = H5Tget_member_name(dstype, idx)             # <<<<<<<<<<<<<<
 *         if name not in mtype_fields:
 *             continue
 */
    __pyx_t_4 = __pyx_f_4h5py_4defs_H5Tget_member_name(__pyx_v_dstype, __pyx_v_idx); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 209; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_name = __pyx_t_4;

    /* "h5py/_proxy.pyx":210
 *     for idx in xrange(H5Tget_nmembers(dstype)):
 *         name = H5Tget_member_name(dstype, idx)
 *         if name not in mtype_fields:             # <<<<<<<<<<<<<<
 *             continue
 *         temptype = H5Tget_member_type(dstype, idx)
 */
    __pyx_t_1 = PyBytes_FromString(__pyx_v_name); if (unlikely(!__pyx_t_1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 210; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(((PyObject *)__pyx_t_1));
    __pyx_t_6 = (__Pyx_PySequence_Contains(((PyObject *)__pyx_t_1), ((PyObject *)__pyx_v_mtype_fields), Py_NE)); if (unlikely(__pyx_t_6 < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 210; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_DECREF(((PyObject *)__pyx_t_1)); __pyx_t_1 = 0;
    if (__pyx_t_6) {

      /* "h5py/_proxy.pyx":211
 *         name = H5Tget_member_name(dstype, idx)
 *         if name not in mtype_fields:
 *             continue             # <<<<<<<<<<<<<<
 *         temptype = H5Tget_member_type(dstype, idx)
 *         H5Tinsert(newtype, name, offset, temptype)
 */
      goto __pyx_L8_continue;
      goto __pyx_L10;
    }
    __pyx_L10:;

    /* "h5py/_proxy.pyx":212
 *         if name not in mtype_fields:
 *             continue
 *         temptype = H5Tget_member_type(dstype, idx)             # <<<<<<<<<<<<<<
 *         H5Tinsert(newtype, name, offset, temptype)
 *         offset += H5Tget_size(temptype)
 */
    __pyx_t_7 = __pyx_f_4h5py_4defs_H5Tget_member_type(__pyx_v_dstype, __pyx_v_idx); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 212; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_temptype = __pyx_t_7;

    /* "h5py/_proxy.pyx":213
 *             continue
 *         temptype = H5Tget_member_type(dstype, idx)
 *         H5Tinsert(newtype, name, offset, temptype)             # <<<<<<<<<<<<<<
 *         offset += H5Tget_size(temptype)
 *         H5Tclose(temptype)
 */
    __pyx_t_9 = __pyx_f_4h5py_4defs_H5Tinsert(__pyx_v_newtype, __pyx_v_name, __pyx_v_offset, __pyx_v_temptype); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 213; __pyx_clineno = __LINE__; goto __pyx_L1_error;}

    /* "h5py/_proxy.pyx":214
 *         temptype = H5Tget_member_type(dstype, idx)
 *         H5Tinsert(newtype, name, offset, temptype)
 *         offset += H5Tget_size(temptype)             # <<<<<<<<<<<<<<
 *         H5Tclose(temptype)
 * 
 */
    __pyx_t_8 = __pyx_f_4h5py_4defs_H5Tget_size(__pyx_v_temptype); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 214; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_offset = (__pyx_v_offset + __pyx_t_8);

    /* "h5py/_proxy.pyx":215
 *         H5Tinsert(newtype, name, offset, temptype)
 *         offset += H5Tget_size(temptype)
 *         H5Tclose(temptype)             # <<<<<<<<<<<<<<
 * 
 *     return newtype
 */
    __pyx_t_9 = __pyx_f_4h5py_4defs_H5Tclose(__pyx_v_temptype); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 215; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_L8_continue:;
  }

  /* "h5py/_proxy.pyx":217
 *         H5Tclose(temptype)
 * 
 *     return newtype             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = __pyx_v_newtype;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("h5py._proxy.make_reduced_type", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_mtype_fields);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_proxy.pyx":220
 * 
 * 
 * cdef void* create_buffer(size_t ipt_size, size_t opt_size, size_t nl) except NULL:             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t final_size
 */

static void *__pyx_f_4h5py_6_proxy_create_buffer(size_t __pyx_v_ipt_size, size_t __pyx_v_opt_size, size_t __pyx_v_nl) {
  size_t __pyx_v_final_size;
  void *__pyx_v_buf;
  void *__pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("create_buffer", 0);

  /* "h5py/_proxy.pyx":225
 *     cdef void* buf
 * 
 *     if ipt_size >= opt_size:             # <<<<<<<<<<<<<<
 *         final_size = ipt_size*nl
 *     else:
 */
  __pyx_t_1 = (__pyx_v_ipt_size >= __pyx_v_opt_size);
  if (__pyx_t_1) {

    /* "h5py/_proxy.pyx":226
 * 
 *     if ipt_size >= opt_size:
 *         final_size = ipt_size*nl             # <<<<<<<<<<<<<<
 *     else:
 *         final_size = opt_size*nl
 */
    __pyx_v_final_size = (__pyx_v_ipt_size * __pyx_v_nl);
    goto __pyx_L3;
  }
  /*else*/ {

    /* "h5py/_proxy.pyx":228
 *         final_size = ipt_size*nl
 *     else:
 *         final_size = opt_size*nl             # <<<<<<<<<<<<<<
 * 
 *     buf = malloc(final_size)
 */
    __pyx_v_final_size = (__pyx_v_opt_size * __pyx_v_nl);
  }
  __pyx_L3:;

  /* "h5py/_proxy.pyx":230
 *         final_size = opt_size*nl
 * 
 *     buf = malloc(final_size)             # <<<<<<<<<<<<<<
 *     if buf == NULL:
 *         raise MemoryError("Failed to allocate conversion buffer")
 */
  __pyx_v_buf = malloc(__pyx_v_final_size);

  /* "h5py/_proxy.pyx":231
 * 
 *     buf = malloc(final_size)
 *     if buf == NULL:             # <<<<<<<<<<<<<<
 *         raise MemoryError("Failed to allocate conversion buffer")
 * 
 */
  __pyx_t_1 = (__pyx_v_buf == NULL);
  if (__pyx_t_1) {

    /* "h5py/_proxy.pyx":232
 *     buf = malloc(final_size)
 *     if buf == NULL:
 *         raise MemoryError("Failed to allocate conversion buffer")             # <<<<<<<<<<<<<<
 * 
 *     return buf
 */
    __pyx_t_2 = PyObject_Call(__pyx_builtin_MemoryError, ((PyObject *)__pyx_k_tuple_2), NULL); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 232; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 232; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L4;
  }
  __pyx_L4:;

  /* "h5py/_proxy.pyx":234
 *         raise MemoryError("Failed to allocate conversion buffer")
 * 
 *     return buf             # <<<<<<<<<<<<<<
 * 
 * # =============================================================================
 */
  __pyx_r = __pyx_v_buf;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("h5py._proxy.create_buffer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_proxy.pyx":244
 *     void* buf
 * 
 * cdef herr_t h5py_scatter_cb(void* elem, hid_t type_id, unsigned ndim,             # <<<<<<<<<<<<<<
 *                 hsize_t *point, void *operator_data) except -1:
 * 
 */

static herr_t __pyx_f_4h5py_6_proxy_h5py_scatter_cb(void *__pyx_v_elem, CYTHON_UNUSED hid_t __pyx_v_type_id, CYTHON_UNUSED unsigned int __pyx_v_ndim, CYTHON_UNUSED hsize_t *__pyx_v_point, void *__pyx_v_operator_data) {
  __pyx_t_4h5py_6_proxy_h5py_scatter_t *__pyx_v_info;
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  __Pyx_RefNannySetupContext("h5py_scatter_cb", 0);

  /* "h5py/_proxy.pyx":247
 *                 hsize_t *point, void *operator_data) except -1:
 * 
 *     cdef h5py_scatter_t* info = <h5py_scatter_t*>operator_data             # <<<<<<<<<<<<<<
 * 
 *     memcpy(elem, (<char*>info[0].buf)+((info[0].i)*(info[0].elsize)),
 */
  __pyx_v_info = ((__pyx_t_4h5py_6_proxy_h5py_scatter_t *)__pyx_v_operator_data);

  /* "h5py/_proxy.pyx":250
 * 
 *     memcpy(elem, (<char*>info[0].buf)+((info[0].i)*(info[0].elsize)),
 *            info[0].elsize)             # <<<<<<<<<<<<<<
 * 
 *     info[0].i += 1
 */
  memcpy(__pyx_v_elem, (((char *)(__pyx_v_info[0]).buf) + ((__pyx_v_info[0]).i * (__pyx_v_info[0]).elsize)), (__pyx_v_info[0]).elsize);

  /* "h5py/_proxy.pyx":252
 *            info[0].elsize)
 * 
 *     info[0].i += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_t_1 = 0;
  (__pyx_v_info[__pyx_t_1]).i = ((__pyx_v_info[__pyx_t_1]).i + 1);

  /* "h5py/_proxy.pyx":254
 *     info[0].i += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * cdef herr_t h5py_gather_cb(void* elem, hid_t type_id, unsigned ndim,
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_proxy.pyx":256
 *     return 0
 * 
 * cdef herr_t h5py_gather_cb(void* elem, hid_t type_id, unsigned ndim,             # <<<<<<<<<<<<<<
 *                 hsize_t *point, void *operator_data) except -1:
 * 
 */

static herr_t __pyx_f_4h5py_6_proxy_h5py_gather_cb(void *__pyx_v_elem, CYTHON_UNUSED hid_t __pyx_v_type_id, CYTHON_UNUSED unsigned int __pyx_v_ndim, CYTHON_UNUSED hsize_t *__pyx_v_point, void *__pyx_v_operator_data) {
  __pyx_t_4h5py_6_proxy_h5py_scatter_t *__pyx_v_info;
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  long __pyx_t_1;
  __Pyx_RefNannySetupContext("h5py_gather_cb", 0);

  /* "h5py/_proxy.pyx":259
 *                 hsize_t *point, void *operator_data) except -1:
 * 
 *     cdef h5py_scatter_t* info = <h5py_scatter_t*>operator_data             # <<<<<<<<<<<<<<
 * 
 *     memcpy((<char*>info[0].buf)+((info[0].i)*(info[0].elsize)), elem,
 */
  __pyx_v_info = ((__pyx_t_4h5py_6_proxy_h5py_scatter_t *)__pyx_v_operator_data);

  /* "h5py/_proxy.pyx":262
 * 
 *     memcpy((<char*>info[0].buf)+((info[0].i)*(info[0].elsize)), elem,
 *             info[0].elsize)             # <<<<<<<<<<<<<<
 * 
 *     info[0].i += 1
 */
  memcpy((((char *)(__pyx_v_info[0]).buf) + ((__pyx_v_info[0]).i * (__pyx_v_info[0]).elsize)), __pyx_v_elem, (__pyx_v_info[0]).elsize);

  /* "h5py/_proxy.pyx":264
 *             info[0].elsize)
 * 
 *     info[0].i += 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_t_1 = 0;
  (__pyx_v_info[__pyx_t_1]).i = ((__pyx_v_info[__pyx_t_1]).i + 1);

  /* "h5py/_proxy.pyx":266
 *     info[0].i += 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * # Copy between a contiguous and non-contiguous buffer, with the layout
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_proxy.pyx":270
 * # Copy between a contiguous and non-contiguous buffer, with the layout
 * # of the latter specified by a dataspace selection.
 * cdef herr_t h5py_copy(hid_t tid, hid_t space, void* contig, void* noncontig,             # <<<<<<<<<<<<<<
 *                  copy_dir op) except -1:
 * 
 */

static herr_t __pyx_f_4h5py_6_proxy_h5py_copy(hid_t __pyx_v_tid, hid_t __pyx_v_space, void *__pyx_v_contig, void *__pyx_v_noncontig, enum __pyx_t_4h5py_6_proxy_copy_dir __pyx_v_op) {
  __pyx_t_4h5py_6_proxy_h5py_scatter_t __pyx_v_info;
  hsize_t __pyx_v_elsize;
  herr_t __pyx_r;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  int __pyx_t_2;
  herr_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("h5py_copy", 0);

  /* "h5py/_proxy.pyx":276
 *     cdef hsize_t elsize
 * 
 *     elsize = H5Tget_size(tid)             # <<<<<<<<<<<<<<
 * 
 *     info.i = 0
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tget_size(__pyx_v_tid); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 276; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_elsize = __pyx_t_1;

  /* "h5py/_proxy.pyx":278
 *     elsize = H5Tget_size(tid)
 * 
 *     info.i = 0             # <<<<<<<<<<<<<<
 *     info.elsize = elsize
 *     info.buf = contig
 */
  __pyx_v_info.i = 0;

  /* "h5py/_proxy.pyx":279
 * 
 *     info.i = 0
 *     info.elsize = elsize             # <<<<<<<<<<<<<<
 *     info.buf = contig
 * 
 */
  __pyx_v_info.elsize = __pyx_v_elsize;

  /* "h5py/_proxy.pyx":280
 *     info.i = 0
 *     info.elsize = elsize
 *     info.buf = contig             # <<<<<<<<<<<<<<
 * 
 *     if op == H5PY_SCATTER:
 */
  __pyx_v_info.buf = __pyx_v_contig;

  /* "h5py/_proxy.pyx":282
 *     info.buf = contig
 * 
 *     if op == H5PY_SCATTER:             # <<<<<<<<<<<<<<
 *         H5Diterate(noncontig, tid, space, h5py_scatter_cb, &info)
 *     elif op == H5PY_GATHER:
 */
  __pyx_t_2 = (__pyx_v_op == __pyx_e_4h5py_6_proxy_H5PY_SCATTER);
  if (__pyx_t_2) {

    /* "h5py/_proxy.pyx":283
 * 
 *     if op == H5PY_SCATTER:
 *         H5Diterate(noncontig, tid, space, h5py_scatter_cb, &info)             # <<<<<<<<<<<<<<
 *     elif op == H5PY_GATHER:
 *         H5Diterate(noncontig, tid, space, h5py_gather_cb, &info)
 */
    __pyx_t_3 = __pyx_f_4h5py_4defs_H5Diterate(__pyx_v_noncontig, __pyx_v_tid, __pyx_v_space, __pyx_f_4h5py_6_proxy_h5py_scatter_cb, (&__pyx_v_info)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 283; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L3;
  }

  /* "h5py/_proxy.pyx":284
 *     if op == H5PY_SCATTER:
 *         H5Diterate(noncontig, tid, space, h5py_scatter_cb, &info)
 *     elif op == H5PY_GATHER:             # <<<<<<<<<<<<<<
 *         H5Diterate(noncontig, tid, space, h5py_gather_cb, &info)
 *     else:
 */
  __pyx_t_2 = (__pyx_v_op == __pyx_e_4h5py_6_proxy_H5PY_GATHER);
  if (__pyx_t_2) {

    /* "h5py/_proxy.pyx":285
 *         H5Diterate(noncontig, tid, space, h5py_scatter_cb, &info)
 *     elif op == H5PY_GATHER:
 *         H5Diterate(noncontig, tid, space, h5py_gather_cb, &info)             # <<<<<<<<<<<<<<
 *     else:
 *         raise RuntimeError("Illegal direction")
 */
    __pyx_t_3 = __pyx_f_4h5py_4defs_H5Diterate(__pyx_v_noncontig, __pyx_v_tid, __pyx_v_space, __pyx_f_4h5py_6_proxy_h5py_gather_cb, (&__pyx_v_info)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 285; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    goto __pyx_L3;
  }
  /*else*/ {

    /* "h5py/_proxy.pyx":287
 *         H5Diterate(noncontig, tid, space, h5py_gather_cb, &info)
 *     else:
 *         raise RuntimeError("Illegal direction")             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
    __pyx_t_4 = PyObject_Call(__pyx_builtin_RuntimeError, ((PyObject *)__pyx_k_tuple_4), NULL); if (unlikely(!__pyx_t_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 287; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    {__pyx_filename = __pyx_f[0]; __pyx_lineno = 287; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  }
  __pyx_L3:;

  /* "h5py/_proxy.pyx":289
 *         raise RuntimeError("Illegal direction")
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * # =============================================================================
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("h5py._proxy.h5py_copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_proxy.pyx":294
 * # VLEN support routines
 * 
 * cdef htri_t needs_bkg_buffer(hid_t src, hid_t dst) except -1:             # <<<<<<<<<<<<<<
 * 
 *     cdef H5T_cdata_t *info = NULL
 */

static htri_t __pyx_f_4h5py_6_proxy_needs_bkg_buffer(hid_t __pyx_v_src, hid_t __pyx_v_dst) {
  H5T_cdata_t *__pyx_v_info;
  htri_t __pyx_r;
  __Pyx_RefNannyDeclarations
  htri_t __pyx_t_1;
  htri_t __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  H5T_conv_t __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  size_t __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  char *__pyx_t_13;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("needs_bkg_buffer", 0);

  /* "h5py/_proxy.pyx":296
 * cdef htri_t needs_bkg_buffer(hid_t src, hid_t dst) except -1:
 * 
 *     cdef H5T_cdata_t *info = NULL             # <<<<<<<<<<<<<<
 * 
 *     if H5Tdetect_class(src, H5T_COMPOUND) or H5Tdetect_class(dst, H5T_COMPOUND):
 */
  __pyx_v_info = NULL;

  /* "h5py/_proxy.pyx":298
 *     cdef H5T_cdata_t *info = NULL
 * 
 *     if H5Tdetect_class(src, H5T_COMPOUND) or H5Tdetect_class(dst, H5T_COMPOUND):             # <<<<<<<<<<<<<<
 *         return 1
 * 
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tdetect_class(__pyx_v_src, H5T_COMPOUND); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (!__pyx_t_1) {
    __pyx_t_2 = __pyx_f_4h5py_4defs_H5Tdetect_class(__pyx_v_dst, H5T_COMPOUND); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 298; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_t_3 = __pyx_t_2;
  } else {
    __pyx_t_3 = __pyx_t_1;
  }
  if (__pyx_t_3) {

    /* "h5py/_proxy.pyx":299
 * 
 *     if H5Tdetect_class(src, H5T_COMPOUND) or H5Tdetect_class(dst, H5T_COMPOUND):
 *         return 1             # <<<<<<<<<<<<<<
 * 
 *     try:
 */
    __pyx_r = 1;
    goto __pyx_L0;
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/_proxy.pyx":301
 *         return 1
 * 
 *     try:             # <<<<<<<<<<<<<<
 *         H5Tfind(src, dst, &info)
 *     except:
 */
  {
    __Pyx_ExceptionSave(&__pyx_t_4, &__pyx_t_5, &__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_6);
    /*try:*/ {

      /* "h5py/_proxy.pyx":302
 * 
 *     try:
 *         H5Tfind(src, dst, &info)             # <<<<<<<<<<<<<<
 *     except:
 *         print "Failed to find converter for %s -> %s" % (H5Tget_size(src), H5Tget_tag(dst))
 */
      __pyx_t_7 = __pyx_f_4h5py_4defs_H5Tfind(__pyx_v_src, __pyx_v_dst, (&__pyx_v_info)); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 302; __pyx_clineno = __LINE__; goto __pyx_L4_error;}
    }
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    goto __pyx_L11_try_end;
    __pyx_L4_error:;

    /* "h5py/_proxy.pyx":303
 *     try:
 *         H5Tfind(src, dst, &info)
 *     except:             # <<<<<<<<<<<<<<
 *         print "Failed to find converter for %s -> %s" % (H5Tget_size(src), H5Tget_tag(dst))
 *         raise
 */
    /*except:*/ {
      __Pyx_AddTraceback("h5py._proxy.needs_bkg_buffer", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_8, &__pyx_t_9, &__pyx_t_10) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 303; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_GOTREF(__pyx_t_10);

      /* "h5py/_proxy.pyx":304
 *         H5Tfind(src, dst, &info)
 *     except:
 *         print "Failed to find converter for %s -> %s" % (H5Tget_size(src), H5Tget_tag(dst))             # <<<<<<<<<<<<<<
 *         raise
 * 
 */
      __pyx_t_11 = __pyx_f_4h5py_4defs_H5Tget_size(__pyx_v_src); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 304; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __pyx_t_12 = __Pyx_PyInt_FromSize_t(__pyx_t_11); if (unlikely(!__pyx_t_12)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 304; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_12);
      __pyx_t_13 = __pyx_f_4h5py_4defs_H5Tget_tag(__pyx_v_dst); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 304; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __pyx_t_14 = PyBytes_FromString(__pyx_t_13); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 304; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(((PyObject *)__pyx_t_14));
      __pyx_t_15 = PyTuple_New(2); if (unlikely(!__pyx_t_15)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 304; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(__pyx_t_15);
      PyTuple_SET_ITEM(__pyx_t_15, 0, __pyx_t_12);
      __Pyx_GIVEREF(__pyx_t_12);
      PyTuple_SET_ITEM(__pyx_t_15, 1, ((PyObject *)__pyx_t_14));
      __Pyx_GIVEREF(((PyObject *)__pyx_t_14));
      __pyx_t_12 = 0;
      __pyx_t_14 = 0;
      __pyx_t_14 = PyNumber_Remainder(((PyObject *)__pyx_kp_s_5), ((PyObject *)__pyx_t_15)); if (unlikely(!__pyx_t_14)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 304; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_GOTREF(((PyObject *)__pyx_t_14));
      __Pyx_DECREF(((PyObject *)__pyx_t_15)); __pyx_t_15 = 0;
      if (__Pyx_PrintOne(0, ((PyObject *)__pyx_t_14)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 304; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_DECREF(((PyObject *)__pyx_t_14)); __pyx_t_14 = 0;

      /* "h5py/_proxy.pyx":305
 *     except:
 *         print "Failed to find converter for %s -> %s" % (H5Tget_size(src), H5Tget_tag(dst))
 *         raise             # <<<<<<<<<<<<<<
 * 
 *     if info[0].need_bkg == H5T_BKG_YES:
 */
      __Pyx_GIVEREF(__pyx_t_8);
      __Pyx_GIVEREF(__pyx_t_9);
      __Pyx_GIVEREF(__pyx_t_10);
      __Pyx_ErrRestore(__pyx_t_8, __pyx_t_9, __pyx_t_10);
      __pyx_t_8 = 0; __pyx_t_9 = 0; __pyx_t_10 = 0; 
      {__pyx_filename = __pyx_f[0]; __pyx_lineno = 305; __pyx_clineno = __LINE__; goto __pyx_L6_except_error;}
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
      goto __pyx_L5_exception_handled;
    }
    __pyx_L6_except_error:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    goto __pyx_L1_error;
    __pyx_L5_exception_handled:;
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_ExceptionReset(__pyx_t_4, __pyx_t_5, __pyx_t_6);
    __pyx_L11_try_end:;
  }

  /* "h5py/_proxy.pyx":307
 *         raise
 * 
 *     if info[0].need_bkg == H5T_BKG_YES:             # <<<<<<<<<<<<<<
 *         return 1
 * 
 */
  __pyx_t_3 = ((__pyx_v_info[0]).need_bkg == H5T_BKG_YES);
  if (__pyx_t_3) {

    /* "h5py/_proxy.pyx":308
 * 
 *     if info[0].need_bkg == H5T_BKG_YES:
 *         return 1             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
    __pyx_r = 1;
    goto __pyx_L0;
    goto __pyx_L14;
  }
  __pyx_L14:;

  /* "h5py/_proxy.pyx":310
 *         return 1
 * 
 *     return 0             # <<<<<<<<<<<<<<
 * 
 * # Determine if the given type requires proxy buffering
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_14);
  __Pyx_XDECREF(__pyx_t_15);
  __Pyx_AddTraceback("h5py._proxy.needs_bkg_buffer", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "h5py/_proxy.pyx":313
 * 
 * # Determine if the given type requires proxy buffering
 * cdef htri_t needs_proxy(hid_t tid) except -1:             # <<<<<<<<<<<<<<
 * 
 *     cdef H5T_class_t cls
 */

static htri_t __pyx_f_4h5py_6_proxy_needs_proxy(hid_t __pyx_v_tid) {
  enum H5T_class_t __pyx_v_cls;
  hid_t __pyx_v_supertype;
  int __pyx_v_i;
  int __pyx_v_n;
  htri_t __pyx_v_result;
  htri_t __pyx_r;
  __Pyx_RefNannyDeclarations
  enum H5T_class_t __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  htri_t __pyx_t_5;
  hid_t __pyx_t_6;
  herr_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("needs_proxy", 0);

  /* "h5py/_proxy.pyx":320
 *     cdef htri_t result
 * 
 *     cls = H5Tget_class(tid)             # <<<<<<<<<<<<<<
 * 
 *     if cls == H5T_VLEN or cls == H5T_REFERENCE:
 */
  __pyx_t_1 = __pyx_f_4h5py_4defs_H5Tget_class(__pyx_v_tid); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 320; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_v_cls = __pyx_t_1;

  /* "h5py/_proxy.pyx":322
 *     cls = H5Tget_class(tid)
 * 
 *     if cls == H5T_VLEN or cls == H5T_REFERENCE:             # <<<<<<<<<<<<<<
 *         return 1
 * 
 */
  __pyx_t_2 = (__pyx_v_cls == H5T_VLEN);
  if (!__pyx_t_2) {
    __pyx_t_3 = (__pyx_v_cls == H5T_REFERENCE);
    __pyx_t_4 = __pyx_t_3;
  } else {
    __pyx_t_4 = __pyx_t_2;
  }
  if (__pyx_t_4) {

    /* "h5py/_proxy.pyx":323
 * 
 *     if cls == H5T_VLEN or cls == H5T_REFERENCE:
 *         return 1             # <<<<<<<<<<<<<<
 * 
 *     elif cls == H5T_STRING:
 */
    __pyx_r = 1;
    goto __pyx_L0;
    goto __pyx_L3;
  }

  /* "h5py/_proxy.pyx":325
 *         return 1
 * 
 *     elif cls == H5T_STRING:             # <<<<<<<<<<<<<<
 *         return H5Tis_variable_str(tid)
 * 
 */
  __pyx_t_4 = (__pyx_v_cls == H5T_STRING);
  if (__pyx_t_4) {

    /* "h5py/_proxy.pyx":326
 * 
 *     elif cls == H5T_STRING:
 *         return H5Tis_variable_str(tid)             # <<<<<<<<<<<<<<
 * 
 *     elif cls == H5T_ARRAY:
 */
    __pyx_t_5 = __pyx_f_4h5py_4defs_H5Tis_variable_str(__pyx_v_tid); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 326; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;
    goto __pyx_L3;
  }

  /* "h5py/_proxy.pyx":328
 *         return H5Tis_variable_str(tid)
 * 
 *     elif cls == H5T_ARRAY:             # <<<<<<<<<<<<<<
 * 
 *         supertype = H5Tget_super(tid)
 */
  __pyx_t_4 = (__pyx_v_cls == H5T_ARRAY);
  if (__pyx_t_4) {

    /* "h5py/_proxy.pyx":330
 *     elif cls == H5T_ARRAY:
 * 
 *         supertype = H5Tget_super(tid)             # <<<<<<<<<<<<<<
 *         try:
 *             return needs_proxy(supertype)
 */
    __pyx_t_6 = __pyx_f_4h5py_4defs_H5Tget_super(__pyx_v_tid); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 330; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_supertype = __pyx_t_6;

    /* "h5py/_proxy.pyx":331
 * 
 *         supertype = H5Tget_super(tid)
 *         try:             # <<<<<<<<<<<<<<
 *             return needs_proxy(supertype)
 *         finally:
 */
    /*try:*/ {

      /* "h5py/_proxy.pyx":332
 *         supertype = H5Tget_super(tid)
 *         try:
 *             return needs_proxy(supertype)             # <<<<<<<<<<<<<<
 *         finally:
 *             H5Tclose(supertype)
 */
      __pyx_t_5 = __pyx_f_4h5py_6_proxy_needs_proxy(__pyx_v_supertype); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 332; __pyx_clineno = __LINE__; goto __pyx_L5;}
      __pyx_r = __pyx_t_5;
      goto __pyx_L4;
    }

    /* "h5py/_proxy.pyx":334
 *             return needs_proxy(supertype)
 *         finally:
 *             H5Tclose(supertype)             # <<<<<<<<<<<<<<
 * 
 *     elif cls == H5T_COMPOUND:
 */
    /*finally:*/ {
      int __pyx_why;
      PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
      int __pyx_exc_lineno;
      __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
      __pyx_why = 0; goto __pyx_L6;
      __pyx_L4: __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
      __pyx_why = 3; goto __pyx_L6;
      __pyx_L5: {
        __pyx_why = 4;
        __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
        __pyx_exc_lineno = __pyx_lineno;
        goto __pyx_L6;
      }
      __pyx_L6:;
      __pyx_t_7 = __pyx_f_4h5py_4defs_H5Tclose(__pyx_v_supertype); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 334; __pyx_clineno = __LINE__; goto __pyx_L7_error;}
      goto __pyx_L8;
      __pyx_L7_error:;
      if (__pyx_why == 4) {
        Py_XDECREF(__pyx_exc_type);
        Py_XDECREF(__pyx_exc_value);
        Py_XDECREF(__pyx_exc_tb);
      }
      goto __pyx_L1_error;
      __pyx_L8:;
      switch (__pyx_why) {
        case 3: goto __pyx_L0;
        case 4: {
          __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
          __pyx_lineno = __pyx_exc_lineno;
          __pyx_exc_type = 0;
          __pyx_exc_value = 0;
          __pyx_exc_tb = 0;
          goto __pyx_L1_error;
        }
      }
    }
    goto __pyx_L3;
  }

  /* "h5py/_proxy.pyx":336
 *             H5Tclose(supertype)
 * 
 *     elif cls == H5T_COMPOUND:             # <<<<<<<<<<<<<<
 * 
 *         n = H5Tget_nmembers(tid)
 */
  __pyx_t_4 = (__pyx_v_cls == H5T_COMPOUND);
  if (__pyx_t_4) {

    /* "h5py/_proxy.pyx":338
 *     elif cls == H5T_COMPOUND:
 * 
 *         n = H5Tget_nmembers(tid)             # <<<<<<<<<<<<<<
 *         for i from 0<=i<n:
 *             supertype = H5Tget_member_type(tid, i)
 */
    __pyx_t_8 = __pyx_f_4h5py_4defs_H5Tget_nmembers(__pyx_v_tid); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 338; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    __pyx_v_n = __pyx_t_8;

    /* "h5py/_proxy.pyx":339
 * 
 *         n = H5Tget_nmembers(tid)
 *         for i from 0<=i<n:             # <<<<<<<<<<<<<<
 *             supertype = H5Tget_member_type(tid, i)
 *             try:
 */
    __pyx_t_8 = __pyx_v_n;
    for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_8; __pyx_v_i++) {

      /* "h5py/_proxy.pyx":340
 *         n = H5Tget_nmembers(tid)
 *         for i from 0<=i<n:
 *             supertype = H5Tget_member_type(tid, i)             # <<<<<<<<<<<<<<
 *             try:
 *                 result = needs_proxy(supertype)
 */
      __pyx_t_6 = __pyx_f_4h5py_4defs_H5Tget_member_type(__pyx_v_tid, __pyx_v_i); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 340; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
      __pyx_v_supertype = __pyx_t_6;

      /* "h5py/_proxy.pyx":341
 *         for i from 0<=i<n:
 *             supertype = H5Tget_member_type(tid, i)
 *             try:             # <<<<<<<<<<<<<<
 *                 result = needs_proxy(supertype)
 *                 if result > 0:
 */
      /*try:*/ {

        /* "h5py/_proxy.pyx":342
 *             supertype = H5Tget_member_type(tid, i)
 *             try:
 *                 result = needs_proxy(supertype)             # <<<<<<<<<<<<<<
 *                 if result > 0:
 *                     return 1
 */
        __pyx_t_5 = __pyx_f_4h5py_6_proxy_needs_proxy(__pyx_v_supertype); if (unlikely(__pyx_t_5 == -1)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 342; __pyx_clineno = __LINE__; goto __pyx_L14;}
        __pyx_v_result = __pyx_t_5;

        /* "h5py/_proxy.pyx":343
 *             try:
 *                 result = needs_proxy(supertype)
 *                 if result > 0:             # <<<<<<<<<<<<<<
 *                     return 1
 *             finally:
 */
        __pyx_t_4 = (__pyx_v_result > 0);
        if (__pyx_t_4) {

          /* "h5py/_proxy.pyx":344
 *                 result = needs_proxy(supertype)
 *                 if result > 0:
 *                     return 1             # <<<<<<<<<<<<<<
 *             finally:
 *                 H5Tclose(supertype)
 */
          __pyx_r = 1;
          goto __pyx_L13;
          goto __pyx_L16;
        }
        __pyx_L16:;
      }

      /* "h5py/_proxy.pyx":346
 *                     return 1
 *             finally:
 *                 H5Tclose(supertype)             # <<<<<<<<<<<<<<
 *         return 0
 * 
 */
      /*finally:*/ {
        int __pyx_why;
        PyObject *__pyx_exc_type, *__pyx_exc_value, *__pyx_exc_tb;
        int __pyx_exc_lineno;
        __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
        __pyx_why = 0; goto __pyx_L15;
        __pyx_L13: __pyx_exc_type = 0; __pyx_exc_value = 0; __pyx_exc_tb = 0; __pyx_exc_lineno = 0;
        __pyx_why = 3; goto __pyx_L15;
        __pyx_L14: {
          __pyx_why = 4;
          __Pyx_ErrFetch(&__pyx_exc_type, &__pyx_exc_value, &__pyx_exc_tb);
          __pyx_exc_lineno = __pyx_lineno;
          goto __pyx_L15;
        }
        __pyx_L15:;
        __pyx_t_7 = __pyx_f_4h5py_4defs_H5Tclose(__pyx_v_supertype); if (unlikely(PyErr_Occurred())) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 346; __pyx_clineno = __LINE__; goto __pyx_L17_error;}
        goto __pyx_L18;
        __pyx_L17_error:;
        if (__pyx_why == 4) {
          Py_XDECREF(__pyx_exc_type);
          Py_XDECREF(__pyx_exc_value);
          Py_XDECREF(__pyx_exc_tb);
        }
        goto __pyx_L1_error;
        __pyx_L18:;
        switch (__pyx_why) {
          case 3: goto __pyx_L0;
          case 4: {
            __Pyx_ErrRestore(__pyx_exc_type, __pyx_exc_value, __pyx_exc_tb);
            __pyx_lineno = __pyx_exc_lineno;
            __pyx_exc_type = 0;
            __pyx_exc_value = 0;
            __pyx_exc_tb = 0;
            goto __pyx_L1_error;
          }
        }
      }
    }

    /* "h5py/_proxy.pyx":347
 *             finally:
 *                 H5Tclose(supertype)
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
    __pyx_r = 0;
    goto __pyx_L0;
    goto __pyx_L3;
  }
  __pyx_L3:;

  /* "h5py/_proxy.pyx":349
 *         return 0
 * 
 *     return 0             # <<<<<<<<<<<<<<
 */
  __pyx_r = 0;
  goto __pyx_L0;

  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("h5py._proxy.needs_proxy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    __Pyx_NAMESTR("_proxy"),
    __Pyx_DOCSTR(__pyx_k_6), /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_1, __pyx_k_1, sizeof(__pyx_k_1), 0, 0, 1, 0},
  {&__pyx_kp_s_3, __pyx_k_3, sizeof(__pyx_k_3), 0, 0, 1, 0},
  {&__pyx_kp_s_5, __pyx_k_5, sizeof(__pyx_k_5), 0, 0, 1, 0},
  {&__pyx_n_s__MemoryError, __pyx_k__MemoryError, sizeof(__pyx_k__MemoryError), 0, 0, 1, 1},
  {&__pyx_n_s__RuntimeError, __pyx_k__RuntimeError, sizeof(__pyx_k__RuntimeError), 0, 0, 1, 1},
  {&__pyx_n_s____main__, __pyx_k____main__, sizeof(__pyx_k____main__), 0, 0, 1, 1},
  {&__pyx_n_s____test__, __pyx_k____test__, sizeof(__pyx_k____test__), 0, 0, 1, 1},
  {&__pyx_n_s__range, __pyx_k__range, sizeof(__pyx_k__range), 0, 0, 1, 1},
  {&__pyx_n_s__xrange, __pyx_k__xrange, sizeof(__pyx_k__xrange), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  #if PY_MAJOR_VERSION >= 3
  __pyx_builtin_xrange = __Pyx_GetName(__pyx_b, __pyx_n_s__range); if (!__pyx_builtin_xrange) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 190; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #else
  __pyx_builtin_xrange = __Pyx_GetName(__pyx_b, __pyx_n_s__xrange); if (!__pyx_builtin_xrange) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 190; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  __pyx_builtin_MemoryError = __Pyx_GetName(__pyx_b, __pyx_n_s__MemoryError); if (!__pyx_builtin_MemoryError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 232; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_builtin_RuntimeError = __Pyx_GetName(__pyx_b, __pyx_n_s__RuntimeError); if (!__pyx_builtin_RuntimeError) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 287; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "h5py/_proxy.pyx":232
 *     buf = malloc(final_size)
 *     if buf == NULL:
 *         raise MemoryError("Failed to allocate conversion buffer")             # <<<<<<<<<<<<<<
 * 
 *     return buf
 */
  __pyx_k_tuple_2 = PyTuple_Pack(1, ((PyObject *)__pyx_kp_s_1)); if (unlikely(!__pyx_k_tuple_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 232; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_2);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_2));

  /* "h5py/_proxy.pyx":287
 *         H5Diterate(noncontig, tid, space, h5py_gather_cb, &info)
 *     else:
 *         raise RuntimeError("Illegal direction")             # <<<<<<<<<<<<<<
 * 
 *     return 0
 */
  __pyx_k_tuple_4 = PyTuple_Pack(1, ((PyObject *)__pyx_kp_s_3)); if (unlikely(!__pyx_k_tuple_4)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 287; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(__pyx_k_tuple_4);
  __Pyx_GIVEREF(((PyObject *)__pyx_k_tuple_4));
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC init_proxy(void); /*proto*/
PyMODINIT_FUNC init_proxy(void)
#else
PyMODINIT_FUNC PyInit__proxy(void); /*proto*/
PyMODINIT_FUNC PyInit__proxy(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit__proxy(void)", 0);
  if ( __Pyx_check_binary_version() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #ifdef __Pyx_CyFunction_USED
  if (__Pyx_CyFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4(__Pyx_NAMESTR("_proxy"), __pyx_methods, __Pyx_DOCSTR(__pyx_k_6), 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    if (!PyDict_GetItemString(modules, "h5py._proxy")) {
      if (unlikely(PyDict_SetItemString(modules, "h5py._proxy", __pyx_m) < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
    }
  }
  #endif
  __pyx_b = PyImport_AddModule(__Pyx_NAMESTR(__Pyx_BUILTIN_MODULE_NAME)); if (unlikely(!__pyx_b)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (__Pyx_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  /*--- Initialize various global constants etc. ---*/
  if (unlikely(__Pyx_InitGlobals() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__pyx_module_is_main_h5py___proxy) {
    if (__Pyx_SetAttrString(__pyx_m, "__name__", __pyx_n_s____main__) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;};
  }
  /*--- Builtin init code ---*/
  if (unlikely(__Pyx_InitCachedBuiltins() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Constants init code ---*/
  if (unlikely(__Pyx_InitCachedConstants() < 0)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  if (__Pyx_ExportFunction("attr_rw", (void (*)(void))__pyx_f_4h5py_6_proxy_attr_rw, "herr_t (hid_t, hid_t, void *, int)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ExportFunction("dset_rw", (void (*)(void))__pyx_f_4h5py_6_proxy_dset_rw, "herr_t (hid_t, hid_t, hid_t, hid_t, hid_t, void *, int)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  /*--- Type init code ---*/
  /*--- Type import code ---*/
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  __pyx_t_1 = __Pyx_ImportModule("h5py.defs"); if (!__pyx_t_1) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Dget_space", (void (**)(void))&__pyx_f_4h5py_4defs_H5Dget_space, "hid_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Dget_type", (void (**)(void))&__pyx_f_4h5py_4defs_H5Dget_type, "hid_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Dread", (void (**)(void))&__pyx_f_4h5py_4defs_H5Dread, "herr_t (hid_t, hid_t, hid_t, hid_t, hid_t, void *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Dwrite", (void (**)(void))&__pyx_f_4h5py_4defs_H5Dwrite, "herr_t (hid_t, hid_t, hid_t, hid_t, hid_t, void *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Dvlen_reclaim", (void (**)(void))&__pyx_f_4h5py_4defs_H5Dvlen_reclaim, "herr_t (hid_t, hid_t, hid_t, void *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Diterate", (void (**)(void))&__pyx_f_4h5py_4defs_H5Diterate, "herr_t (void *, hid_t, hid_t, H5D_operator_t, void *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sclose", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sclose, "herr_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Screate_simple", (void (**)(void))&__pyx_f_4h5py_4defs_H5Screate_simple, "hid_t (int, hsize_t *, hsize_t *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Sget_select_npoints", (void (**)(void))&__pyx_f_4h5py_4defs_H5Sget_select_npoints, "hssize_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tcreate", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tcreate, "hid_t (enum H5T_class_t, size_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tget_class", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tget_class, "enum H5T_class_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tget_size", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tget_size, "size_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tget_super", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tget_super, "hid_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tdetect_class", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tdetect_class, "htri_t (hid_t, enum H5T_class_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tclose", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tclose, "herr_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tconvert", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tconvert, "herr_t (hid_t, hid_t, size_t, void *, void *, hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tis_variable_str", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tis_variable_str, "htri_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tget_nmembers", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tget_nmembers, "int (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tget_member_name", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tget_member_name, "char *(hid_t, unsigned int)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tget_member_type", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tget_member_type, "hid_t (hid_t, unsigned int)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tinsert", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tinsert, "herr_t (hid_t, char *, size_t, hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tget_tag", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tget_tag, "char *(hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Tfind", (void (**)(void))&__pyx_f_4h5py_4defs_H5Tfind, "H5T_conv_t (hid_t, hid_t, H5T_cdata_t **)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Aread", (void (**)(void))&__pyx_f_4h5py_4defs_H5Aread, "herr_t (hid_t, hid_t, void *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Awrite", (void (**)(void))&__pyx_f_4h5py_4defs_H5Awrite, "herr_t (hid_t, hid_t, void *)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Aget_space", (void (**)(void))&__pyx_f_4h5py_4defs_H5Aget_space, "hid_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  if (__Pyx_ImportFunction(__pyx_t_1, "H5Aget_type", (void (**)(void))&__pyx_f_4h5py_4defs_H5Aget_type, "hid_t (hid_t)") < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  Py_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  /*--- Execution code ---*/

  /* "h5py/_proxy.pyx":1
 * # cython: profile=False             # <<<<<<<<<<<<<<
 * 
 * # This file is part of h5py, a Python interface to the HDF5 library.
 */
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_GOTREF(((PyObject *)__pyx_t_2));
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s____test__, ((PyObject *)__pyx_t_2)) < 0) {__pyx_filename = __pyx_f[0]; __pyx_lineno = 1; __pyx_clineno = __LINE__; goto __pyx_L1_error;}
  __Pyx_DECREF(((PyObject *)__pyx_t_2)); __pyx_t_2 = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    __Pyx_AddTraceback("init h5py._proxy", __pyx_clineno, __pyx_lineno, __pyx_filename);
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init h5py._proxy");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* Runtime support code */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif /* CYTHON_REFNANNY */

static PyObject *__Pyx_GetName(PyObject *dict, PyObject *name) {
    PyObject *result;
    result = PyObject_GetAttr(dict, name);
    if (!result) {
        if (dict != __pyx_b) {
            PyErr_Clear();
            result = PyObject_GetAttr(__pyx_b, name);
        }
        if (!result) {
            PyErr_SetObject(PyExc_NameError, name);
        }
    }
    return result;
}

static CYTHON_INLINE void __Pyx_ErrRestore(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_Restore(type, value, tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetch(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(type, value, tb);
#endif
}

#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    #if PY_VERSION_HEX < 0x02050000
    if (PyClass_Check(type)) {
    #else
    if (PyType_Check(type)) {
    #endif
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        #if PY_VERSION_HEX < 0x02050000
            if (PyInstance_Check(type)) {
                type = (PyObject*) ((PyInstanceObject*)type)->in_class;
                Py_INCREF(type);
            }
            else {
                type = 0;
                PyErr_SetString(PyExc_TypeError,
                    "raise: exception must be an old-style class or instance");
                goto raise_error;
            }
        #else
            type = (PyObject*) Py_TYPE(type);
            Py_INCREF(type);
            if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
                PyErr_SetString(PyExc_TypeError,
                    "raise: exception class must be a subclass of BaseException");
                goto raise_error;
            }
        #endif
    }
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else /* Python 3+ */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *args;
        if (!value)
            args = PyTuple_New(0);
        else if (PyTuple_Check(value)) {
            Py_INCREF(value);
            args = value;
        }
        else
            args = PyTuple_Pack(1, value);
        if (!args)
            goto bad;
        owned_instance = PyEval_CallObject(type, args);
        Py_DECREF(args);
        if (!owned_instance)
            goto bad;
        value = owned_instance;
        if (!PyExceptionInstance_Check(value)) {
            PyErr_Format(PyExc_TypeError,
                         "calling %R should have returned an instance of "
                         "BaseException, not %R",
                         type, Py_TYPE(value));
            goto bad;
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause && cause != Py_None) {
        PyObject *fixed_cause;
        if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        }
        else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        }
        else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_COMPILING_IN_CPYTHON
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
        goto bad;
    #endif
    Py_INCREF(local_type);
    Py_INCREF(local_value);
    Py_INCREF(local_tb);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_COMPILING_IN_CPYTHON
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    /* Make sure tstate is in a consistent state when we XDECREF
       these objects (DECREF may run arbitrary code). */
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

static CYTHON_INLINE void __Pyx_ExceptionSave(PyObject **type, PyObject **value, PyObject **tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyThreadState *tstate = PyThreadState_GET();
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
#else
    PyErr_GetExcInfo(type, value, tb);
#endif
}
static void __Pyx_ExceptionReset(PyObject *type, PyObject *value, PyObject *tb) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyThreadState *tstate = PyThreadState_GET();
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(type, value, tb);
#endif
}

#if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION < 3
static PyObject *__Pyx_GetStdout(void) {
    PyObject *f = PySys_GetObject((char *)"stdout");
    if (!f) {
        PyErr_SetString(PyExc_RuntimeError, "lost sys.stdout");
    }
    return f;
}
static int __Pyx_Print(PyObject* f, PyObject *arg_tuple, int newline) {
    int i;
    if (!f) {
        if (!(f = __Pyx_GetStdout()))
            return -1;
    }
    Py_INCREF(f);
    for (i=0; i < PyTuple_GET_SIZE(arg_tuple); i++) {
        PyObject* v;
        if (PyFile_SoftSpace(f, 1)) {
            if (PyFile_WriteString(" ", f) < 0)
                goto error;
        }
        v = PyTuple_GET_ITEM(arg_tuple, i);
        if (PyFile_WriteObject(v, f, Py_PRINT_RAW) < 0)
            goto error;
        if (PyString_Check(v)) {
            char *s = PyString_AsString(v);
            Py_ssize_t len = PyString_Size(v);
            if (len > 0 &&
                isspace(Py_CHARMASK(s[len-1])) &&
                s[len-1] != ' ')
                    PyFile_SoftSpace(f, 0);
        }
    }
    if (newline) {
        if (PyFile_WriteString("\n", f) < 0)
            goto error;
        PyFile_SoftSpace(f, 0);
    }
    Py_DECREF(f);
    return 0;
error:
    Py_DECREF(f);
    return -1;
}
#else /* Python 3 has a print function */
static int __Pyx_Print(PyObject* stream, PyObject *arg_tuple, int newline) {
    PyObject* kwargs = 0;
    PyObject* result = 0;
    PyObject* end_string;
    if (unlikely(!__pyx_print)) {
        __pyx_print = __Pyx_GetAttrString(__pyx_b, "print");
        if (!__pyx_print)
            return -1;
    }
    if (stream) {
        kwargs = PyDict_New();
        if (unlikely(!kwargs))
            return -1;
        if (unlikely(PyDict_SetItemString(kwargs, "file", stream) < 0))
            goto bad;
        if (!newline) {
            end_string = PyUnicode_FromStringAndSize(" ", 1);
            if (unlikely(!end_string))
                goto bad;
            if (PyDict_SetItemString(kwargs, "end", end_string) < 0) {
                Py_DECREF(end_string);
                goto bad;
            }
            Py_DECREF(end_string);
        }
    } else if (!newline) {
        if (unlikely(!__pyx_print_kwargs)) {
            __pyx_print_kwargs = PyDict_New();
            if (unlikely(!__pyx_print_kwargs))
                return -1;
            end_string = PyUnicode_FromStringAndSize(" ", 1);
            if (unlikely(!end_string))
                return -1;
            if (PyDict_SetItemString(__pyx_print_kwargs, "end", end_string) < 0) {
                Py_DECREF(end_string);
                return -1;
            }
            Py_DECREF(end_string);
        }
        kwargs = __pyx_print_kwargs;
    }
    result = PyObject_Call(__pyx_print, arg_tuple, kwargs);
    if (unlikely(kwargs) && (kwargs != __pyx_print_kwargs))
        Py_DECREF(kwargs);
    if (!result)
        return -1;
    Py_DECREF(result);
    return 0;
bad:
    if (kwargs != __pyx_print_kwargs)
        Py_XDECREF(kwargs);
    return -1;
}
#endif

#if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION < 3
static int __Pyx_PrintOne(PyObject* f, PyObject *o) {
    if (!f) {
        if (!(f = __Pyx_GetStdout()))
            return -1;
    }
    Py_INCREF(f);
    if (PyFile_SoftSpace(f, 0)) {
        if (PyFile_WriteString(" ", f) < 0)
            goto error;
    }
    if (PyFile_WriteObject(o, f, Py_PRINT_RAW) < 0)
        goto error;
    if (PyFile_WriteString("\n", f) < 0)
        goto error;
    Py_DECREF(f);
    return 0;
error:
    Py_DECREF(f);
    return -1;
    /* the line below is just to avoid C compiler
     * warnings about unused functions */
    return __Pyx_Print(f, NULL, 0);
}
#else /* Python 3 has a print function */
static int __Pyx_PrintOne(PyObject* stream, PyObject *o) {
    int res;
    PyObject* arg_tuple = PyTuple_Pack(1, o);
    if (unlikely(!arg_tuple))
        return -1;
    res = __Pyx_Print(stream, arg_tuple, 1);
    Py_DECREF(arg_tuple);
    return res;
}
#endif

static CYTHON_INLINE unsigned char __Pyx_PyInt_AsUnsignedChar(PyObject* x) {
    const unsigned char neg_one = (unsigned char)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(unsigned char) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(unsigned char)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to unsigned char" :
                    "value too large to convert to unsigned char");
            }
            return (unsigned char)-1;
        }
        return (unsigned char)val;
    }
    return (unsigned char)__Pyx_PyInt_AsUnsignedLong(x);
}

static CYTHON_INLINE unsigned short __Pyx_PyInt_AsUnsignedShort(PyObject* x) {
    const unsigned short neg_one = (unsigned short)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(unsigned short) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(unsigned short)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to unsigned short" :
                    "value too large to convert to unsigned short");
            }
            return (unsigned short)-1;
        }
        return (unsigned short)val;
    }
    return (unsigned short)__Pyx_PyInt_AsUnsignedLong(x);
}

static CYTHON_INLINE unsigned int __Pyx_PyInt_AsUnsignedInt(PyObject* x) {
    const unsigned int neg_one = (unsigned int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(unsigned int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(unsigned int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to unsigned int" :
                    "value too large to convert to unsigned int");
            }
            return (unsigned int)-1;
        }
        return (unsigned int)val;
    }
    return (unsigned int)__Pyx_PyInt_AsUnsignedLong(x);
}

static CYTHON_INLINE char __Pyx_PyInt_AsChar(PyObject* x) {
    const char neg_one = (char)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(char) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(char)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to char" :
                    "value too large to convert to char");
            }
            return (char)-1;
        }
        return (char)val;
    }
    return (char)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE short __Pyx_PyInt_AsShort(PyObject* x) {
    const short neg_one = (short)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(short) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(short)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to short" :
                    "value too large to convert to short");
            }
            return (short)-1;
        }
        return (short)val;
    }
    return (short)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE int __Pyx_PyInt_AsInt(PyObject* x) {
    const int neg_one = (int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to int" :
                    "value too large to convert to int");
            }
            return (int)-1;
        }
        return (int)val;
    }
    return (int)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE signed char __Pyx_PyInt_AsSignedChar(PyObject* x) {
    const signed char neg_one = (signed char)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(signed char) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(signed char)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to signed char" :
                    "value too large to convert to signed char");
            }
            return (signed char)-1;
        }
        return (signed char)val;
    }
    return (signed char)__Pyx_PyInt_AsSignedLong(x);
}

static CYTHON_INLINE signed short __Pyx_PyInt_AsSignedShort(PyObject* x) {
    const signed short neg_one = (signed short)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(signed short) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(signed short)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to signed short" :
                    "value too large to convert to signed short");
            }
            return (signed short)-1;
        }
        return (signed short)val;
    }
    return (signed short)__Pyx_PyInt_AsSignedLong(x);
}

static CYTHON_INLINE signed int __Pyx_PyInt_AsSignedInt(PyObject* x) {
    const signed int neg_one = (signed int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(signed int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(signed int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to signed int" :
                    "value too large to convert to signed int");
            }
            return (signed int)-1;
        }
        return (signed int)val;
    }
    return (signed int)__Pyx_PyInt_AsSignedLong(x);
}

static CYTHON_INLINE int __Pyx_PyInt_AsLongDouble(PyObject* x) {
    const int neg_one = (int)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
    if (sizeof(int) < sizeof(long)) {
        long val = __Pyx_PyInt_AsLong(x);
        if (unlikely(val != (long)(int)val)) {
            if (!unlikely(val == -1 && PyErr_Occurred())) {
                PyErr_SetString(PyExc_OverflowError,
                    (is_unsigned && unlikely(val < 0)) ?
                    "can't convert negative value to int" :
                    "value too large to convert to int");
            }
            return (int)-1;
        }
        return (int)val;
    }
    return (int)__Pyx_PyInt_AsLong(x);
}

static CYTHON_INLINE unsigned long __Pyx_PyInt_AsUnsignedLong(PyObject* x) {
    const unsigned long neg_one = (unsigned long)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to unsigned long");
            return (unsigned long)-1;
        }
        return (unsigned long)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to unsigned long");
                return (unsigned long)-1;
            }
            return (unsigned long)PyLong_AsUnsignedLong(x);
        } else {
            return (unsigned long)PyLong_AsLong(x);
        }
    } else {
        unsigned long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (unsigned long)-1;
        val = __Pyx_PyInt_AsUnsignedLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE unsigned PY_LONG_LONG __Pyx_PyInt_AsUnsignedLongLong(PyObject* x) {
    const unsigned PY_LONG_LONG neg_one = (unsigned PY_LONG_LONG)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to unsigned PY_LONG_LONG");
            return (unsigned PY_LONG_LONG)-1;
        }
        return (unsigned PY_LONG_LONG)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to unsigned PY_LONG_LONG");
                return (unsigned PY_LONG_LONG)-1;
            }
            return (unsigned PY_LONG_LONG)PyLong_AsUnsignedLongLong(x);
        } else {
            return (unsigned PY_LONG_LONG)PyLong_AsLongLong(x);
        }
    } else {
        unsigned PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (unsigned PY_LONG_LONG)-1;
        val = __Pyx_PyInt_AsUnsignedLongLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE long __Pyx_PyInt_AsLong(PyObject* x) {
    const long neg_one = (long)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to long");
            return (long)-1;
        }
        return (long)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to long");
                return (long)-1;
            }
            return (long)PyLong_AsUnsignedLong(x);
        } else {
            return (long)PyLong_AsLong(x);
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (long)-1;
        val = __Pyx_PyInt_AsLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE PY_LONG_LONG __Pyx_PyInt_AsLongLong(PyObject* x) {
    const PY_LONG_LONG neg_one = (PY_LONG_LONG)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to PY_LONG_LONG");
            return (PY_LONG_LONG)-1;
        }
        return (PY_LONG_LONG)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to PY_LONG_LONG");
                return (PY_LONG_LONG)-1;
            }
            return (PY_LONG_LONG)PyLong_AsUnsignedLongLong(x);
        } else {
            return (PY_LONG_LONG)PyLong_AsLongLong(x);
        }
    } else {
        PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (PY_LONG_LONG)-1;
        val = __Pyx_PyInt_AsLongLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE signed long __Pyx_PyInt_AsSignedLong(PyObject* x) {
    const signed long neg_one = (signed long)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to signed long");
            return (signed long)-1;
        }
        return (signed long)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to signed long");
                return (signed long)-1;
            }
            return (signed long)PyLong_AsUnsignedLong(x);
        } else {
            return (signed long)PyLong_AsLong(x);
        }
    } else {
        signed long val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (signed long)-1;
        val = __Pyx_PyInt_AsSignedLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static CYTHON_INLINE signed PY_LONG_LONG __Pyx_PyInt_AsSignedLongLong(PyObject* x) {
    const signed PY_LONG_LONG neg_one = (signed PY_LONG_LONG)-1, const_zero = 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_VERSION_HEX < 0x03000000
    if (likely(PyInt_Check(x))) {
        long val = PyInt_AS_LONG(x);
        if (is_unsigned && unlikely(val < 0)) {
            PyErr_SetString(PyExc_OverflowError,
                            "can't convert negative value to signed PY_LONG_LONG");
            return (signed PY_LONG_LONG)-1;
        }
        return (signed PY_LONG_LONG)val;
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
            if (unlikely(Py_SIZE(x) < 0)) {
                PyErr_SetString(PyExc_OverflowError,
                                "can't convert negative value to signed PY_LONG_LONG");
                return (signed PY_LONG_LONG)-1;
            }
            return (signed PY_LONG_LONG)PyLong_AsUnsignedLongLong(x);
        } else {
            return (signed PY_LONG_LONG)PyLong_AsLongLong(x);
        }
    } else {
        signed PY_LONG_LONG val;
        PyObject *tmp = __Pyx_PyNumber_Int(x);
        if (!tmp) return (signed PY_LONG_LONG)-1;
        val = __Pyx_PyInt_AsSignedLongLong(tmp);
        Py_DECREF(tmp);
        return val;
    }
}

static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
}

static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        #if PY_VERSION_HEX < 0x02050000
        return PyErr_Warn(NULL, message);
        #else
        return PyErr_WarnEx(NULL, message, 1);
        #endif
    }
    return 0;
}

static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(__pyx_m, (char *)"__pyx_capi__");
    if (!d) {
        PyErr_Clear();
        d = PyDict_New();
        if (!d)
            goto bad;
        Py_INCREF(d);
        if (PyModule_AddObject(__pyx_m, (char *)"__pyx_capi__", d) < 0)
            goto bad;
    }
    tmp.fp = f;
#if PY_VERSION_HEX >= 0x02070000 && !(PY_MAJOR_VERSION==3&&PY_MINOR_VERSION==0)
    cobj = PyCapsule_New(tmp.p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(tmp.p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItemString(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

#ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

#ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%s does not export expected C function %s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000 && !(PY_MAJOR_VERSION==3 && PY_MINOR_VERSION==0)
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %s.%s has wrong signature (expected %s, got %s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %s.%s has wrong signature (expected %s, got %s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = (start + end) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,            /*int argcount,*/
        0,            /*int kwonlyargcount,*/
        0,            /*int nlocals,*/
        0,            /*int stacksize,*/
        0,            /*int flags,*/
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,      /*int firstlineno,*/
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_globals = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_globals = PyModule_GetDict(__pyx_m);
    if (!py_globals) goto bad;
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        py_globals,          /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    py_frame->f_lineno = py_line;
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else  /* Python 3+ has unicode identifiers */
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Int(PyObject* x) {
  PyNumberMethods *m;
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_VERSION_HEX < 0x03000000
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return Py_INCREF(x), x;
  m = Py_TYPE(x)->tp_as_number;
#if PY_VERSION_HEX < 0x03000000
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
#else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
#endif
  if (res) {
#if PY_VERSION_HEX < 0x03000000
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%s__ returned non-%s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject* x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
#if PY_VERSION_HEX < 0x02050000
   if (ival <= LONG_MAX)
       return PyInt_FromLong((long)ival);
   else {
       unsigned char *bytes = (unsigned char *) &ival;
       int one = 1; int little = (int)*(unsigned char*)&one;
       return _PyLong_FromByteArray(bytes, sizeof(size_t), little, 0);
   }
#else
   return PyInt_FromSize_t(ival);
#endif
}
static CYTHON_INLINE size_t __Pyx_PyInt_AsSize_t(PyObject* x) {
   unsigned PY_LONG_LONG val = __Pyx_PyInt_AsUnsignedLongLong(x);
   if (unlikely(val == (unsigned PY_LONG_LONG)-1 && PyErr_Occurred())) {
       return (size_t)-1;
   } else if (unlikely(val != (unsigned PY_LONG_LONG)(size_t)val)) {
       PyErr_SetString(PyExc_OverflowError,
                       "value too large to convert to size_t");
       return (size_t)-1;
   }
   return (size_t)val;
}


#endif /* Py_PYTHON_H */
